function CTSbasics
%CTSBASICS A basic version of a clinical trial simulator.
%   CTSBASICS conducts a single-dose placebo-controlled trial for 
%   a drug that acts on the production of a biomarker according to  
%   an indirect response model.

clc; clear; close all;
tic
rng(0,'twister')    % for reproducibility
nrep = 5;
hfresp = figure; 
set(hfresp,'units','normalized','Position',[0.1, 0.4, 0.3, 0.4])
p(1:nrep) = 0;  % to initialize vector p
for rep = 1:nrep
    rep         %#ok<NOPRT>
    nsubjects            = 20;
    [de1,de1r,de2]       = fixedEffects(nsubjects);
    par                  = parameters(de1,de1r,de2);
    [dosage,dosingTimes] = treatments();
    obs                  = observations();
    results              = PSmodel(par,dosage,obs,dosingTimes);
    p(rep)               = analysis(dosage,results);
end
yl = ylim;
ylim([yl(1)-1 yl(2)])
xlabel('Time')
ylabel('Drug Concentration                Response')
legend('placebo','treatment', 'Location', 'Best')
print('-dtiff','-r600','CTSbasics')
outcomes(p)
display(['p = ', num2str(p)])
toc

    function results = PSmodel(par, dosage, obs, dosingTimes) 
    %PSMODEL Conduct simulations.
    %   RESULTS = PSMODEL(PAR, DOSAGE, OBS, DOSINGTIMES) conducts
    %   simulations for all subjects based on the following input
    %   variables: |PAR| - model parameters, |DOSAGE| - dosing
    %   regimen, |OBS| - observations and |DOSINGTIMES| - dose times. 
    %   It returns the matrix |RESULTS|.
    %   The function is nested due to |nsubjects|. 
    
    tlast = max([obs.var1, obs.var2, dosingTimes]);
    intInt = [dosingTimes tlast];   % integration intervals
    results = [];
    for treatment = 1:length(dosage)
        dose = dosage(treatment);
        erg = [];
        for i = 1:nsubjects
            ti   = []; 
            yi   = [];
            ergt = [];
            ergy = [];
            ys = [dosage(treatment); 0; par.rin(i)/par.kout(i)];
            for ni = 2:length(intInt)
                t1 = intInt(ni-1);
                t2 = intInt(ni);
                sol = ode45(@derivatives, [t1,t2], ys, [], par);
                graphics(treatment, sol)
                % sampling times in integration interval
                idx1 = obs.var1 >= t1 & obs.var1 <= t2;
                idx2 = obs.var2 >= t1 & obs.var2 <= t2;
                idx = [idx1 idx2];
                if sum(idx) > 0 
                    t = unique([obs.var1(idx1) obs.var2(idx2)])';
                    y = deval(sol,t)';      
                    ti = [ti; t];       %#ok<AGROW>
                    yi = [yi; y];       %#ok<AGROW>
                end
                ys = sol.y(:,end)' + [dosage(treatment) 0 0];
            end
            ergt = [ergt; ti];          %#ok<AGROW>
            ergy = [ergy; yi];          %#ok<AGROW>
            ergn = [ones(length(ergt),1)*dose, ... 
                ones(length(ergt),1)*i, ergt, ergy];
            erg = [erg; ergn];          %#ok<AGROW>
        end
        results = [results; erg];       %#ok<AGROW>
    end
        
        function dydt = derivatives(~ , y, par)
        %DERIVATIVES Compute the right-hand side of the ODE.
        %   DYDT = DERIVATIVES(T, Y, PAR) calculates |DYDT|, the
        %   right-hand side of the ODE model, at points defined by
        %   the vector of dependent variables |Y|, time |T|, and 
        %   with parameters |PAR|.
        %   The function is nested due to the index |i|.
        
        ka    = par.ka(i);
        ke    = par.ke(i);
        V     = par.V(i);
        rin   = par.rin(i);
        kout  = par.kout(i);
        emax  = par.emax(i);
        ec50  = par.ec50(i);
        a0    = y(1);
        a1    = y(2);
        r     = y(3);
        c     = a1/V;
        de    = hillEffect(c, 1, emax, ec50, 1);
        da0dt = -ka*a0;
        da1dt = ka*a0 - ke* a1;
        drdt  = rin - kout*de*r;
        dydt  = [da0dt; da1dt; drdt];
        
        end 
        
        function graphics(treatment, sol)
        %GRAPHICS Graphs with the drug concentration and response. 
        %   GRAPHICS(TREATMENT, SOL) produces a graph which shows the
        %   time courses of the drug concentration and drug response 
        %   for the selected |TREATMENT| and values given in the 
        %   model solution structure |SOL|.
        
        %   The function is nested with regard to |i| and |ni|

        if treatment == 1 
            hp1 = plot(sol.x, sol.y(2,:), '-b'); drawnow;
            hold on
            hp2 = plot(sol.x, sol.y(3,:), '-b'); drawnow;
        end
        if treatment == 2 
            hp1 = plot(sol.x,sol.y(2,:), '-r'); drawnow; 
            hold on
            hp2 = plot(sol.x,sol.y(3,:), '-r'); drawnow;
        end
        if i ~= 1 || ni ~= 2  
            set(get(get(hp1,'Annotation'),'LegendInformation'),...
            'IconDisplayStyle','off'); % Exclude line from legend
        end
        set(get(get(hp2,'Annotation'),'LegendInformation'),...
        'IconDisplayStyle','off'); % Exclude line from legend
        end

    end
end

function obs = observations
%OBSERVATIONS Specify observation times  
%   OBS = OBSERVATIONS creates a stucture, |OBS|, with observation 
%   times for all dependent variables regarded in the analysis.

obs.var1 = 0:4:24;      
obs.var2 = 0:24:72;

end


function [dosage, dosingTimes] = treatments
%TREATMENTS Dosing specification.
%   [DOSAGE, DOSINGTIMES] = TREATMENTS specifies the dose times 
%   |DOSINGTIMES| and dose amounts |DOSAGE| for considered regimens.

% 2 regimens assumed
dosage(1) = 0 ;             % placebo
dosage(2) = 4 ;             % active drug treatment with 4 mg
dosingTimes = 0:12:48;

end


function par = parameters(demo1, demo1ref, demo2)
%PARAMETERS Create initial values for model parameters.
%   PAR = PARAMETERS(DEMO1, DEMO1REF, DEMO2) returns a structure 
%   |PAR| containing initial values for model parameters, including
%   variability. The values are created for all simulated subjects.
%   |DEMO1|, |DEMO1REF|, |DEMO2| - covariates 1, reference for 
%   covariate 1 and covariate 2, respectively.

n        = length(demo1);
tvka     = 0.2;  % 1/h
omka     = 0.2; 
par.ka   = tvka*exp(normrnd(0,omka,n,1));   % individual value of ka
tvke     = 0.05;
omke     = 0.16;
demo1ke  = 0.85;        % covariate effect of demo1 on ke
par.ke   = tvke.*(demo1./demo1ref).^(demo1ke) ... 
    .*exp(normrnd(0,omke,n,1));
tvV      = 25;
omV      = 0.25;
par.V    = tvV*exp(normrnd(0,omV,n,1));
tvrin    = 10;
omrin    = 0.09;
demo2rin = 0.85;
par.rin  = tvrin*demo2rin.^demo2.*exp(normrnd(0,omrin,n,1));
tvkout   = 0.5;
omkout   = 0.0;
par.kout = tvkout*exp(normrnd(0,omkout,n,1));
tvemax   = -0.90;
omemax   = 0.04;
par.emax = tvemax*exp(normrnd(0,omemax,n,1));
tvec50   = 1.5;
omec50   = 0.16;
par.ec50 = tvec50*exp(normrnd(0,omec50,n,1));

end
 

function [de1, de1r, de2] = fixedEffects(n)
%FIXEDEFFECTS Generate demographics.
%   [DE1, DE1R, DE2] = FIXEDEFFECTS(N) generates the demographic data 
%   for all subjects included in the trial simulation. The number of
%   subjects is |N|. Two types of the data are generated: 1 - normal,
%   |DE1| with the reference value |DE1R|, and 2  binary, |DE2|. 
%   The normal type values represents such covariates as body weight, 
%   height etc.; binary stands for covariates such as sex (F/M), 
%   smokers (YES/NO), etc. 

% type: 'truncated normal'
de1r   = 40;
de1min = 20;
de1max = 60;
de1sd  = 5;
de1 = truncnorm(n,de1r,de1sd,de1min,de1max);
% type: 'binary';
de2 = binornd(1,0.5,n,1);

end


function p = analysis(dosage, results)
%ANALYSIS Compare effects of the active drug and placebo
%   P = ANALYSIS(DOSAGE,RESULTS) compares a group treated with the 
%   active drug to the placebo group using the statistical t-test to  
%   verify the null-hypothesis (no statistically significant effect  
%   of the drug) against the alternative hypothesis (the drug effect
%   is significant). Input parameters: |DOSAGE| is a vector with dose  
%   values for both groups, |RESULTS| - drug effect values needed for
%   the comparison. The returned value |P| is a probability of the  
%   drug effect value under the null hypothesis (the p-value).
 
placebo = results(results(:,1)==dosage(1) & results(:,3)==72,6);
verum   = results(results(:,1)==dosage(2) & results(:,3)==72,6);
[~, p, ~] = ttest2(verum,placebo,[],[],'unequal');

end


function outcomes(p)
%OUTCOMES Show the results of the CTS power calculation. 
%   OUTCOMES(P) produces a graph which shows how the p-values from 
%   all, |P|, are distributed against p = 0.05 (significance level
%   for acceptance or rejection of the hypothesis H0)


power = sum(p<0.05)/length(p);
hfpower = figure;
hapower = axes;
set(hfpower,'units','normalized', ... 
    'Position',[0.1, 0.1, 0.3, 0.15], 'PaperPositionMode','auto')
stem(p,ones(length(p),1))
hold on
stem(0.05,1,'-r')
set(hapower, 'YTickLabel',['';'';''], 'YTick', [])
xlabel(' ')
legend('p-value','0.05', 'Orientation','horizontal')
title(['CTS: power = ', num2str(power)])
display(['CTS: power = ', num2str(power)])
print('-dtiff','-r600','CTSpower')

end


function out = truncnorm(n, mean, sd, min, max)
%TRUNCNORM Random vector from a truncated normal distribution.
%   OUT = TRUNCNORM(N, MEAN, SD, MIN, MAX) returns a vector of normal 
%   distributed random numbers |OUT|, with the mean value |MEAN| and    
%   standard deviation |SD|, and truncated to the interval with 
%   lower cutoff |MIN| and upper cutoff |MAX|. |N| is the length of 
%   |OUT|.

res = normrnd(mean, sd, n, 1);
fi = find(res < min | res > max);
lf = length(fi);
while lf > 0 
    res1 = normrnd(mean, sd, lf, 1);
    res(fi) = res1;
    fi = find(res < min | res > max);
    lf = length(fi);
end
out = res;

end


function E = hillEffect(c, E0, Emax, EC50, n)
%HILLEFFECT Compute drug effect based on the Hill equation
%   E = HILLEFFECT(C, E0, EMAX, EC50, N) computes drug effect |E|, 
%   based on the Hill equation as a function of concentration |C|,
%   and with the following concentration-response parameters:
%   |E0| - baseline response, |EMAX| - maximum effect, 
%   |EC50| - concentration related to 50% of max. effect, 
%   and |N| - Hill coefficient of sigmoidicity.

E = E0 + Emax.*c.^n./(EC50+c.^n); 

end
