function DDEmar
%DDEMAR solves a PK model with time delay
%   DDEMAR is a simple example that solves a PK model equation 
%   described in form of a DDE. The computation is repeated for 
%   several delay values. For comparison, the solutions are plotted  
%   in the same figure.  

% PK Parameters
p.F  = 1;
p.CL = 2.16;
p.V  = 12.6;
p.ka = 0.1;
% parameters for calling |dde23| 
p.c0 = 0;              % init. concentration
p.A0 = 100;            % init. drug amount [microgram]
tspan = [0 60];            
options = ddeset('RelTol',1e-6, 'AbsTol', 1e-6);
alag = {3 2 1};       % possible delays
a = axes; set(a,'FontSize', 15)
for si = struct('alag', alag, 'col', {'b', 'g', 'r'})
    sol  = dde23(@derivatives, si.alag, @history, tspan, options, p);
    t  = sol.x; 
    y  = sol.y;
    plot(t, (y(2,:)),si.col, 'LineWidth', 2); hold on 
end
% compare with analytical solution without delay (alag=0)
Dose = p.A0; F = p.F; ke = p.CL/p.V; ka = p.ka; V = p.V;
yana = (F*Dose/V)*(ka/(ka-ke))*(exp(-ke*t) - exp(-ka*t));
plot(t, yana, 'k', 'LineWidth', 2); 

legend(['t_{lag}=' num2str(alag{1})],['t_{lag}='       ...
    num2str(alag{2})], ['t_{lag}=' num2str(alag{3})],  ... 
    't_{lag}=0', 'Location', 'Best')
title('Concentration-Time Course');
xlabel('Time [h]')
ylabel('Concentration [\mug/L]')
print('-r900', '-dtiff', 'ddemar')

end        
 

function dydt = derivatives(~, y, Z, p)
%DERIVATIVES Compute the right-hand side of the DDE.
%   DYDT = DERIVATIVES(T, Y, Z, P) calculates |DYDT|, he right-hand
%   side of the DDE model, at points defined by the vector |Y|, |T|,
%   and with parameters |P|. Matrix |Z| corresponds to all possible
%   delays and contains solutions |Y| at the delayed argument. Thus,
%   |Z| has the number of rows equal to the length of |Y|, and the
%   number of column equal to the number of delays. For example:
%   Z(2,1) means y(2) at time |T|-clag; clag is a delay.

% PK Model
a = y(1);              % amount
%c = y(2);             % y(2) at t is not used directly
clag = Z(2,1);         % y(2) at delayed time point (t-clag)
dadt = -p.ka*a;
dcdt = (p.ka/p.V/p.F*a - p.CL/p.V*clag);
dydt = [ dadt        
         dcdt ];      
end


function yhist = history(~, p)
%HISTORY Provide the history of the solution 
%   YHIST = HISTORY(T, P) specifies the time profiles of dependent
%   variables at times |T| prior to the initial point. This function
%   is a form of 'initial condition' for the DDE. |P| stands for
%   additional parameters which can be passed to the function.


yhist = [p.A0 p.c0];

end


