function DiseaseProgression
%DISEASEPROGRESSION Generate time courses of untreated diseases.
%   DISEASEPROGRESSION computes time courses and create graphs for
%   four untreated diseases; A: acute disease with spontaneous
%   remission (e.g. seasonal influenza), B: stable chronic disease
%   (e.g. cataract), C: progressive disease with acute episodic
%   attacks (e.g. multiple sclerosis), D: steadily progressive
%   disease (e.g. Alzheimers disease). The function is linked to
%   Fig. 5.38 (in the book).

t = 0:0.01:40;

% C: progressive disease with acute episodic attacks
p.kin = 0.3;
p.kout = 0.1;
[t, y] = ode45(@derivatives, t, 0 ,[], p);
y = y + exp(-(t-6).^2) + 1.1*exp(-(t-12).^2) + 1.2*exp(-(t-18).^2)+ 1.5*exp(-(t-24).^2);
subplot(2,2,3);
plot(t,y(:,1))
xlabel('Time [arbitrary units]', 'FontSize',11)
ylabel('Disease Status [arbitrary units]', 'FontSize',11)
axis([0 40 0 6])
text(3,5,'C')

% D: steadily progressive disease
p.kin = 0.3;
p.kout = 0.06;
[t, y] = ode45(@derivatives, t, 0 , [], p);
subplot(2,2,4);
plot(t,y(:,1))
xlabel('Time [arbitrary units]', 'FontSize',11)
ylabel('Disease Status [arbitrary units]', 'FontSize',11)
axis([0 40 0 6])
text(3,5,'D')

% A: acute disease with spontaneous
y = 3*exp(-(t-3).^2);
subplot(2,2,1);
plot(t,y(:,1))
xlabel('Time [arbitrary units]', 'FontSize',11)
ylabel('Disease Status [arbitrary units]', 'FontSize',11)
axis([0 40 0 6])
text(3,5,'A')

% B: stable chronic disease
p.kin = 3;
p.kout = 1;
[t, y] = ode45(@derivatives, t, 0, [], p);
subplot(2,2,2);
plot(t,y(:,1))
axis([0 40 0 6])
xlabel('Time [arbitrary units]', 'FontSize',11)
ylabel('Disease Status [arbitrary units]', 'FontSize',11)
text(3,5,'B')

print('-dtiff','-r900','disease progression')

end


function dydt = derivatives(~, y, p)
%DERIVATIVES Compute the right-hand side of the ODE.
%   DYDT = DERIVATIVES(T, Y, P) calculates |DYDT|, the right-hand
%   side of the ODE model, at points defined by the vector of
%   dependent variables |Y|, and time |T|, and with parameters |P|.

% disease model
S = y(1);
dS = p.kin - p.kout*S;
dydt = dS;

end




