function DiseaseProgressionTreatment
%DISEASEPROGRESSIONTREATMENT Symptomatic and disease-modifying
%treatment effects.
%   DISEASEPROGRESSIONTREATMENT shows an example of symptomatic and
%   disease-modifying treatment effects for a chronically
%   deteriorating disease in comparison to the disease progression
%   without treatment. Treatment was administered from time = 10 to
%   time = 20.

p.rin = 0.08;
p.k = 0.1;
p.rindrug = 10;
p.kdrug = 3;
p.emax = 4;
p.ec50 = 2;

p.rindrug = 0;
p.symp = 0;
p.prot = 0;
[t, y] = ode45(@derivatives, [0 40], [0 0], [], p);
set(axes,'FontSize', 15)
plot(t,y(:,2),'k', 'LineWidth',2)
hold on
axis([0 40 0.0 1])

p.rindrug = 1;
p.symp = 1;
p.prot = 0;
[t, y] = ode45(@derivatives, [0 40], [0 0], [], p);
plot(t,y(:,2),'b','LineWidth',2)
hold on

p.symp = 0;
p.prot = 1;
p.rindrug = 1;
[t, y] = ode45(@derivatives, [0 40], [0 0], [], p);
plot(t,y(:,2),'r','LineWidth',2)

p.rindrug = 0;
[t, y] = ode45(@derivatives, [0 40], [0 0], [], p);
plot(t,y(:,2),'k','LineWidth',2 )
axis([0 40 0.0 1])
xlabel('Time [arbitrary units]')
ylabel('Disease status')
title('Chronically Deteriorating Disease') 

legend('Disease progression - no treatment', ...
'Symptomatic treatment effect',  ...
'Disease modifying treatment effect', 'Location', 'SouthEast')
       
print('-dtiff', '-r800', 'DiseaseProgressionTreatment')

end

function dydt = derivatives(t, y, p)
%DERIVATIVES Compute the right-hand side of the ODE.
%   DYDT = DERIVATIVES(T, Y, P) calculates |DYDT|, the right-hand
%   side of the ODE model, at points defined by the vector of
%   dependent variables |Y|, and time |T|, and with parameters |P|.

drug     = y(1);
disease  = y(2);
de       = hillEffect(drug, 0, p.emax, p.ec50, 1);
ddrug    = p.rindrug*(t>=10)*(t<=20) - p.kdrug * drug; 
ddisease = p.rin*(1-p.prot*de) - (1+p.symp*10*(t>=10)*(t<=10.5))*p.k*disease; 
dydt = [ddrug ; ddisease];

end

function E = hillEffect(c, E0, Emax, EC50, n)
%HILLEFFECT Compute drug effect based on the Hill equation.
%   E = HILLEFFECT(C, E0, EMAX, EC50, N) computes drug effect |E|,
%   based on the Hill equation as a function of independent variable
%   |C|, and with the following response parameters: |E0| - baseline
%   response, |EMAX| - maximum effect, |EC50| - |C| value related to
%   50% of max. effect, and |N| - Hill coefficient of sigmoidicity.

E = E0 + Emax.*c.^n./(EC50.^n+c.^n);

end