function PKPDIndirectResponse()
%PKPDINDIRECTRESPONSE Solve indirect response model. 
%   PKPDINDIRECTRESPONSE computes the concentration, drug
%   effect, and response variable in a PK-PD indirect response 
%   model with impact on a physiologic mechanism by the drug effect.
%   The model definition contains the Mass option. 

% parameters
p.ka = 2;
p.k = 1.5;
p.V = 2;
p.Emax = 2;
p.EC50 = 1;
p.rin = 2;
p.kout = 1; 

% DGL and variable assignment
options = odeset; 
options.Mass = [1 0 0 0 0; 0 1 0 0 0; 0 0 0 0 0; 0 0 0 0 0; ...  
    0 0 0 0 1];                 
inits = [10,0,0,0,p.rin/p.kout];
[time,y] = ode15s(@derivalgeb,[0 8],inits,options,p);
conc = y(:,3);
E = y(:,4);
R = y(:,5);

figure('Units','characters','Position', [10 10 150 25], ... 
    'PaperPositionMode','auto');
subplot(1,2,1)
plot(time,conc,'k');    % concentration
hold on
plot(time,E,'b');       % drug effect
plot(time,R,'r');       % respone
xlabel('Time','FontSize',15);
legend('Location','Best','c','E','R');
title('PK-PD Indirect Response', 'FontSize',15);

subplot(1,2,2)
plot(conc,R,'k')
xlabel('Plasma Concentration','FontSize',15);
ylabel('Response','FontSize',15);
title('PK-PD Indirect Response','FontSize',15);
text(1.5,2.7,'\leftarrow','Rotation',200,'FontSize',22);
text(1.0,6.0,'\leftarrow','Rotation',320,'FontSize',22);
print('-dtiff','-r900','PKPDIndirectResponse.tif')

end

function dydt = derivalgeb(~,y,p)
%DERIVALGEB Compute the right-hand side of the DAE.
%   DYDT = DERIVALGEB(T, Y, P) calculates |DYDT|, the right-hand
%   side of the DAE model at points defined by the vector of
%   dependent variables |Y|, time |T|, and with parameters |P|. 
%   The right-hand side represents both the differential equations 
%   and algebraic equations.

a0  = y(1);
a1  = y(2);
c   = y(3);                 % concentration
E   = y(4);                 % drug effect
R   = y(5);                 % response
da0 = -p.ka*a0;
da1 = p.ka*a0 - p.k*a1;
dc  = c - a1/p.V;
dE  = E - p.Emax*c/(c+p.EC50);
dR  = p.rin - (p.kout-E)*R;
dydt = [da0; da1; dc; dE; dR];

end
