function consAxes
%CONSAXES Include and handle axes.  

figure(1)
axesA = axes; 
set(axesA,'FontSize',15);
xlabel('Time'); ylabel('Concentration'); title('V => 16 [L]')
ylim([0 1])
hold on

figure(2)
axesB = axes; 
set(axesB,'FontSize',15);
xlabel('Time'); ylabel('Concentration'); title('V < 16 [L]')
ylim([0 1])
hold on;

set([figure(1); figure(2)], 'units', 'normalized', ...
    {'Position'}, {[0.1, 0.5, 0.3, 0.4]; [0.6, 0.5, 0.3, 0.4]});

ns = 50;
p = modelparams(ns);
a0 = 10; c0 = 0; y0 = [a0; c0]; tspan = [0 15];
sparams = struct('ka', num2cell(p.ka'),  ...
                 'V',  num2cell(p.V'),   ... 
                 'F',  num2cell(p.F'),   ...
                 'CL', num2cell(p.CL'));
for pind = sparams
    [t,y] = ode15s(@derivatives, tspan, y0, [], pind);
    if pind.V >= 16
%         figure(1); plot( t, y(:,2), 'k', 'LineWidth',2);  % FIG
        plot(axesA, t, y(:,2), 'k', 'LineWidth',1.5);       % AXES
    else
%         figure(2); plot( t, y(:,2), 'k', 'LineWidth',2);  % FIG
        plot(axesB, t, y(:,2), 'k', 'LineWidth',1.5);       % AXES
    end
pause(1/2)
end
print(figure(1), '-r800', '-dtiff', 'consAxesA') 
print(figure(2), '-r800', '-dtiff', 'consAxesB') 

end




function p = modelparams(ns)
%MODELPARAMS Specification of model parameters.
%   P = MODELPARAMS(NS) creates a structure |P| storing the model 
%   parameters for |NS| subjects.
 
% Reset the random number generator to the default
rng default
p.numSub = ns;                  % number of subjects
% specify model parameters for all subjects
p.ka = 1 + rand(ns,1);          % absorption rates 
p.V = 15 + rand(ns,1)*2;        % volume of distribution 
p.F = abs(1 - rand(ns,1)*0.5);  % bioavailability 
p.CL = 5 + rand(ns,1);          % clearance

end


function dydt = derivatives(~, y, p)
%DERIVATIVES Compute the right-hand side of the ODE.
%   DYDT = DERIVATIVES(T,Y,P) calculates |DYDT|, the right-hand side
%   of the ODE model, at points defined by the vector of dependent
%   variable |Y|, time |T|, and with parameters |P|, specified for
%   one subject.
%
%   See also ode15s

% set dependent variables for all subjects at ones (vectorization)
a = y(1);    % drug amount                                
c = y(2);    % concentration                                  
% PK Model
dadt = -p.ka*a;
dcdt = (p.ka/p.V/p.F*a - p.CL/p.V*c);
% Derivatives vector of ODE system
dydt = [ dadt        
         dcdt ];  
end


