function consecutive
%CONSECUTIVE Conduct trials consecutively.

tic
ns = 500;
p = modelparams(ns); 
a0 = 10; c0 = 0; y0 = [a0; c0]; tspan = [0 15];
set(axes, 'FontSize',15)
for id = 1:p.numSub 
    pind.ka = p.ka(id);
    pind.V = p.V(id);
    pind.F = p.F(id);
    pind.CL = p.CL(id);
    [t,y] = ode15s(@derivatives, tspan, y0, [], pind);
    plot(t,y(:,2),'Color','b'); hold on
end
toc
xlabel('Time'); ylabel('Concentration'); title('Consecutive')
print('-r800', '-dtiff', 'consecutive') 

end


function p = modelparams(ns)
%MODELPARAMS Specification of model parameters.
%   P = MODELPARAMS(NS) creates a structure |P| storing the model 
%   parameters for |NS| subjects.
 
% Reset the random number generator to the default
rng default
p.numSub = ns;                  % number of subjects
% specify model parameters for all subjects
p.ka = 1 + rand(ns,1);          % absorption rates 
p.V = 15 + rand(ns,1)*2;        % volume of distribution 
p.F = abs(1 - rand(ns,1)*0.5);  % bioavailability 
p.CL = 5 + rand(ns,1);          % clearance

end


function dydt = derivatives(~, y, p)
%DERIVATIVES Compute the right-hand side of the ODE.
%   DYDT = DERIVATIVES(T,Y,P) calculates |DYDT|, the right-hand side
%   of the ODE model, at points defined by the vector of dependent
%   variable |Y|, time |T|, and with parameters |P|.
%
%   See also ode15s

% set dependent variables (for one subject)
a = y(1);    % drug amount                                
c = y(2);    % concentration                                  
% PK Model
dadt = -p.ka*a;
dcdt = (p.ka/p.V/p.F*a - p.CL/p.V*c);
% Derivatives vector of ODE system
dydt = [ dadt        
         dcdt ];  
end


