function [doseAdapt, lastAdapt] = doseAdaptation(p, y) 
%DOSEADAPTATION Perform dose adaptation.
%   [DOSEADAPT LASTADAPT] = DOSEADAPTATION(P, Y) specifies dose 
%   changes in terms of absolute Hb values, Hb values relative 
%   to baseline, and Hb values observed within any 4-weeks period. 
%   Dose changes could start at each dosing time (1x/2 weeks), but 
%   once the first dose adaptation has been performed the next dose 
%   adaptations can't occur more often than 1x/4 weeks. |Y| stands
%   for independent variables in the model, and |P| - a structure 
%   with parameters needed for validation of the adaptation rules.
%   This function returns |DOSEADAPT|- vector with values showing  
%   how the doses were changed, and |LASTADAPT| - vector with times 
%   when the doses were changed; both regarding all subjects.

adaptRule = (p.doseTimes(p.doseNext) - p.lastAdapt) >= 28; % 4-weeks  
doseAdapt = p.doseTrue; 
Hb = y(end, 2*p.numSub+1 : 3*p.numSub);
% Hemoglobin difference from baseline - levels
deltaHb = Hb - p.Hb0;
a =                 deltaHb < -2;
b = -2 <= deltaHb & deltaHb < -1; 
c = -1 <= deltaHb & deltaHb <  1;
d =  1 <= deltaHb & deltaHb <  2;
e =  2 <= deltaHb ;
% dose adaption to hemoglobin concentration - levels
doseAdapt(             Hb <   8      ) = 1;
doseAdapt( 8   <= Hb & Hb <   9      ) = 1;
doseAdapt( 9   <= Hb & Hb <  10.5    ) = 1;
doseAdapt(10.5 <= Hb & Hb <= 13.5 & a) = 1;
doseAdapt(10.5 <= Hb & Hb <= 13.5 & b) = 1;
doseAdapt(10.5 <= Hb & Hb <= 13.5 & c) = 1;
doseAdapt(10.5 <= Hb & Hb <= 13.5 & d) = 1;
doseAdapt(10.5 <= Hb & Hb <= 13.5 & e) = 1;
doseAdapt(13.5 <  Hb & Hb <= 14      ) = 1; 
doseAdapt(14   <  Hb                 ) = 1;
% save adaptation flag and adaptation time for each subject 
doseAdapt = doseAdapt.*adaptRule + ~adaptRule;
lastAdapt = p.doseTimes(p.doseNext)*adaptRule + ... 
    p.lastAdapt.*(doseAdapt == 1);
end


