function emaxDAE()
%EMAXDAE
%   EMAXDAE() schedules PK-PD events, using a DAE solver. Drug  
%   effect should be observed within a given observation time. 
%   Infusion rate is a control variable.

% PK-PD parameters
p.Rate = 1;     % infusion rate (mg/h)
p.V    = 6;     % volume of distribution (L)
p.k    = 0.1;   % elimination rate constant (1/h)   
p.Emax = 20;    % max effect (in Hill equation)
p.E50  = 10;    % drug conc. linked to 50% of maximum effect (mg/l)
 
% Calling ODE
M = [ 1 0       % specifies Mass matrix                                               
      0 0 ];                                                   
options = odeset('Mass', M, 'RelTol',1e-6, 'AbsTol', 1e-6);
tspan = [0 25];
% Initial value for the dependent variables
c0 = 0;            
E0 = hillEffect(c0, 0, p.Emax, p.E50, 1);
[t,c] = ode15s(@derivalgeb, tspan, [c0 E0], options, p); 
% Generate graphs 
set(axes,'FontSize',14)
plot(t, c(:,1), 'Color', 'k', 'LineWidth', 3)             
hold on;
plot(t, c(:,2), 'Color', 'b', 'LineWidth', 3) 
plot(t, rate(t, p),'Color', 'r', 'LineWidth',2)
grid minor; grid on ;
xlabel('Time [h]');
ylabel({'Drug Effect', 'Concentration [mg/L]', ... 
    'Infusion Rate [mg/h]'});
legend('conc','effect','rate')
% save graphs as TIFF file 
print('-r600', '-dtiff', 'emaxDAE')
      
end


function massdydt = derivalgeb(t, y, p)                                 
%DERIVALGEB Compute the right-hand side of the DAE.
%   MASSDYDT = DERIVALGEB(T, Y, P) calculates |DYDT|, the right-hand
%   side of the DAE model at points defined by the vector of
%   dependent variables |Y|, time |T|, and with parameters |P|. 
%   The right-hand side represents both the differential equations 
%   and algebraic equations.

massdydt = [rate(t, p)/p.V - p.k*y(1);
            hillEffect(y(1),0,p.Emax,p.E50,1) - y(2)];                           
end


function  r = rate(t, p)                            
%RATE specify infusion rate within observation time
%   R = RATE(T,P) returns the infusion rate |R| at time |T|, and with
%   parameters |P|.

tR = [0 1; 8 9; 15 16; 22 23]';  % rate times (begin end; ... )
r = 0;
for ir = tR
    r = r + p.Rate*(heaviside(t-ir(1)) - heaviside(t-ir(2)));
end

end


function E = hillEffect(c, E0, Emax, EC50, n)
%HILLEFFECT Compute drug effect based on the Hill equation.
%   E = HILLEFFECT(C, E0, EMAX, EC50, N) computes drug effect |E|, 
%   based on the Hill equation as a function of concentration |C|,
%   and with the following concentration-response parameters:
%   |E0| - baseline response, |EMAX| - maximum effect, 
%   |EC50| - concentration related to 50% of max. effect, 
%   and |N| - Hill coefficient of sigmoidicity.

E = E0 + Emax.*c.^n./(EC50.^n+c.^n);

end



   