function emaxevent()
%EMAXEVENT
%   EMAXEVENT() schedules PK-PD events. The drug effect should be 
%   kept within a given range by infusion rate as control variable, 
%   according to the rule: if the drug effect achieves the allowed
%   maximum, then decrease the infusion; if the drug effect  
%   declines to the allowed minimum - infusion has to be increased.  

% PKPD parameters
p.Rate = 1.0;       % infusion rate
p.V    = 6;         % volume of distribution;
p.k    = 0.1;       % elimination rate   
p.Emax = 20;        % max effect (in Hill equation)
p.E50  = 10;        % concentration linked to 50% of max effect
% Calling ODE
options = odeset('Events', @fevents);
tspan = [0 150];
c0 = 0;            % Initial value for the dependent variable
ie = 99;
set(axes,'FontSize',14)
while ~isempty(ie)
    [t,c,te,ye,ie] = ode45(@derivatives, tspan, c0, options,p); 
    % Generate graph  
    plot(t, p.Rate*ones(length(t),1),'Color', 'r', 'LineWidth',3)
    hold on;
    plot(t, c(:,1), 'Color', 'k', 'LineWidth', 3); 
    hold on;
    plot(t, hillEffect(c(:,1), 0, p.Emax, p.E50, 1),  ...
        'Color', 'b', 'LineWidth', 3)
    tspan = [te(end) 500];
    c0 = ye(end,1);
    R1 =  p.Rate;
    % investigate events
    switch ie(end)
        case 1
            % drug effect E has increased to 2  
            % stop infusion 
            p.Rate = 0;
        case 2
            % drug effect E has decreased to 1   
            % restart infusion    
            p.Rate = 1;        % [mg/h]
        case 3
            % end of observation  
            ie = [];        % no more events         
    end
    R2 = p.Rate;
    plot([t(end) t(end)], [R1 R2],'Color', 'r', 'LineWidth',2)
end
grid minor; grid on ;
xlabel('Time [h]');
ylabel({'Drug Effect', 'Concentration [mg/L]', ... 
    'Infusion Rate [mg/h]'});
legend('rate','conc','effect')
title('')
% save graphs as TIFF file 
print('-r900', '-dtiff', 'emaxevent')
   
end


function dcdt = derivatives(~, c, p)       
%DERIVATIVES Compute the right-hand side of the ODE.
% DYDT = DERIVATIVES(T, C, P) calculates |DCDT|, the right-hand
% side of the ODE model, at points defined by the vector of
% dependent variables |C|, time |T|, and with parameters |P|.

dcdt = p.Rate/p.V - p.k*c(1);       
                        
end


function [value,isterminal,direction] = fevents(t,y,p)
%FEVENTS Specify events for the ODE model.
% [VALUE,ISTERMINAL,DIRECTION] = FEVENTS(T,Y,P) detects the exact
% time points of events specified as a function of time |T|,
% vector of dependent variables |Y|, and a structure of some )
% (specific) other parameters |P|.
%
% Output parameters: |VALUE| - value investigated as event;
% |ISTERMINAL| = 0|1 where 0 stands for 'don't stop', 1 for 'stop
% integration'; |DIRECTION| = -1|0|1 where (-1) means 'negative
% direction only', 0 - 'both direction', 1 - 'positive direction
% only'

% Event 1: Drug effect E has increased to the limit (upper)
value(1,1) = hillEffect(y(1), 0, p.Emax, p.E50, 1) - 0.5; 
isterminal(1,1) = 1;          % Stop integration
direction(1,1)  = 1;          % Positive direction only

% Event 2: Drug effect E has increased to the limit (lower)
value(2,1) = hillEffect(y(1), 0, p.Emax, p.E50, 1) - 0.3;
isterminal(2,1) =  1;         % Stop integration
direction(2,1)  = -1;         % Negative direction only

% Event 3: End of time integration 
value(3,1) = t - 25 ;   
isterminal(3,1) = 1;          % Stop integration
direction(3,1)  = 1;          % Positive direction only

end 


function E = hillEffect(c, E0, Emax, EC50, n)
%HILLEFFECT Compute drug effect based on the Hill equation.
%   E = HILLEFFECT(C, E0, EMAX, EC50, N) computes drug effect |E|, 
%   based on the Hill equation as a function of concentration |C|,
%   and with the following concentration-response parameters:
%   |E0| - baseline response, |EMAX| - maximum effect, 
%   |EC50| - concentration related to 50% of max. effect, 
%   and |N| - Hill coefficient of sigmoidicity.

E = E0 + Emax.*c.^n./(EC50+c.^n); 

end


