function emaxeventDAE()
%EMAXEVENTDAE
%   EMAXEVENTDAE() schedules PKPD events, using a DAE solver. The  
%   drug effect should be kept within a given range by the infusion 
%   rate (control variable) according to the rule: if the drug 
%   effect achieves the allowed maximum, then decrease infusion; if 
%   the drug effect declines to allowed minimum - infusion has to be
%   increased. 

% PKPD parameters
p.R    = 1.0;      % infusion rate
p.V    = 6;        % volume of distribution
p.k    = 0.1;      % elimination rate   
p.Emax = 20;       % max effect (in Hill equation)
p.E50  = 10;       % concentration linked to 50% of max effect
% Calling ODE
M = [ 1 0                                                       %EXT 
      0 0 ];                                                    %EXT
options = odeset('Events', @fevents, 'Mass', M);                %EXT
tspan = [0 150];
c0 = 0;            % Initial value for the dependent variable
E0 = hillEffect(c0, 0, p.Emax, p.E50, 1);
ie = 99;
set(axes,'FontSize',14)
while ~isempty(ie)
    [t,c,te,ye,ie] = ode15s(@derivalgeb, tspan, [c0 E0], ...   %CHA
                     options,p); 
    % Generate graph  
    plot(t, p.R*ones(length(t),1),'Color', 'r', 'LineWidth',2)
    hold on;
    plot(t, c(:,1), 'Color', 'k', 'LineWidth', 3);             
    hold on;
    plot(t, c(:,2), 'Color', 'b', 'LineWidth', 3)               %CHA
    tspan = [te(end) 500];
    c0 = ye(end,1);
    E0 = ye(end,2);
    R1 =  p.R;                                                  %CHA
    % investigate events
    switch ie(end)
        case 1
            % drug effect E has increased to 2  
            % stop infusion 
            p.R = 0;
        case 2
            % drug effect E has decreased to 1   
            % restart infusion    
            p.R = 1;        % [mg/h]
        case 3
            % end of observation  
            ie = [];        % no more events         
    end
    R2 = p.R;
    plot([t(end) t(end)], [R1 R2],'Color', 'r', 'LineWidth',2)
end
grid minor; grid on ;
xlabel('Time [h]');
ylabel({'Drug Effect', 'Concentration [mg/L]', ... 
    'Infusion Rate [mg/h]'});
legend('rate','conc','effect')
title('DAE Solution')
% save graphs as TIFF file 
print('-r900', '-dtiff', 'emaxeventDAE')
   
end


function dydt = derivalgeb(~, y, p)                             %CHA      
%FEVENTS Specify events for the ODE model.
% [VALUE,ISTERMINAL,DIRECTION] = FEVENTS(T,Y,P) detects the exact
% time points of events specified as a function of time |T|,
% vector of dependent variables |Y|, and a structure of some )
% (specific) other parameters |P|.
%
% Output parameters: |VALUE| - value investigated as event;
% |ISTERMINAL| = 0|1 where 0 stands for 'don't stop', 1 for 'stop
% integration'; |DIRECTION| = -1|0|1 where (-1) means 'negative
% direction only', 0 - 'both direction', 1 - 'positive direction
% only'

dydt = [p.R/p.V - p.k*y(1);
        hillEffect(y(1), 0, p.Emax, p.E50, 1) - y(2)];          %EXT
end


function [value,isterminal,direction] = fevents(t,y,~)
%FEVENTS Define events.
%   [VALUE,ISTERMINAL,DIRECTION] = ...  
   
% Event: Drug effect E has increased to the limit (upper)
value(1,1) = y(2) - 0.5;                                        %CHA 
isterminal(1,1) = 1;          % Stop integration
direction(1,1)  = 1;          % Positive direction only

% Event: Drug effect E has increased to the limit (lower)
value(2,1) = y(2) - 0.3;                                        %CHA
isterminal(2,1) =  1;         % Stop integration
direction(2,1)  = -1;         % Negative direction only

% Event: End of time integration 
value(3,1) = t - 25 ;                                           %CHA
isterminal(3,1) = 1;          % Stop integration
direction(3,1)  = 1;          % Positive direction only

end 
 

function E = hillEffect(c, E0, Emax, EC50, n)
%HILLEFFECT Compute drug effect based on the Hill equation.
%   E = HILLEFFECT(C, E0, EMAX, EC50, N) computes drug effect |E|, 
%   based on the Hill equation as a function of concentration |C|,
%   and with the following concentration-response parameters:
%   |E0| - baseline response, |EMAX| - maximum effect, 
%   |EC50| - concentration related to 50% of max. effect, 
%   and |N| - Hill coefficient of sigmoidicity.

E = E0 + Emax.*c.^n./(EC50+c.^n); 

end




   