function epoDDE01
%EPODDE01 Dynamic of Hb in a physiological model.
%   EPODDE01 shows how to implement a patho-physiological model as      
%   a DDE system. It produces a graph showing the Hb time profile  
%   in the case of renal anaemia.

options = ddeset('RelTol',1e-6,'AbsTol',1e-6,'InitialY',10);
p.LS = 100;
p.hb0 = 11;
p.hb00 = 14;
p.tstop = 250;
p.start = 20;
p.beta = 0.02;
p.rin0 = 7.5;
p.stim = 0;
sol = dde23(@derivatives, p.LS, p.hb0, [0 p.tstop], options, p);
set(axes,'FontSize', 15)
plot(sol.x,sol.y,'LineWidth',2)
hold on
rin = prod(sol.x, sol.y, 0, p);
tlin = linspace(p.LS, p.tstop, 1001);
rout = prod(tlin, deval(sol,tlin-p.LS), p.LS,p);
plot(sol.x,rin,'-g','LineWidth',2)
plot(tlin,rout,'-r','LineWidth',2)
plot([0 100],[p.rin0 p.rin0],'-r','LineWidth',2)
legend('Hb [g/dL]','Hb production [g/d]','Hb elimination [g/d]', ...
    'Location', 'East')
xlabel('Time [d]')
print('-dtiff', '-r900', 'Eporegulation')

end


function dydt = derivatives(t, y, Z, p)
%DERIVATIVES Compute the right-hand side of the DDE.
%   DYDT = DERIVATIVES(T, Y, Z, P) calculates |DYDT|, he right-hand
%   side of the DDE model, at points defined by the vector |Y|, |T|,
%   and with parameters |P|. Matrix |Z| corresponds to all possible
%   delays and contains solutions |Y| at the delayed argument. Thus,
%   |Z| has the number of rows equal to the length of |Y|, and the
%   number of column equal to the number of delays. For example:
%   Z(3,1) means y(3) at time |T|-lag; lag is a delay.

Hb = y(1);
lagHb = Z(1,1);
dHB = prod(t, Hb, 0, p) - prod(t, lagHb, p.LS, p);
dydt = dHB;

end


function prod = prod(t,Hb,delay,p)
%PROD Evaluate Hb production.
%   PROD = PROD(T,HB,DELAY,P) computes the hemoglobin production.
%   Input parameters: |T| - time, |HB| - current value of hemoglobin,
%   |DELAY| - life span of erythrocydes, |P| - structure for
%   additional parameters.

tt = t-delay;
prod = (p.rin0 + p.stim*heaviside(tt-p.start)) ... 
    .*exp(p.beta*(p.hb00-Hb).*(delay==0|tt>0));

end


