function epodosing
%EPODOSING investigate the multiple dosing of an erythropoiesis drug  
%   EPODOSING uses a DDE model applied to an erythropoiesis drug. 
%   It allows investigation of dosing regimens, and evaluation of Hb 
%   based adaptive dosing rules.

p.hb0   = 110;
p.LS    = 40;
p.hb00  = 110; % history
p.tstop = 80;
p.start = 20;
p.beta  = 0.;
p.rin0  = 8;
p.s0    = 8;
p.smax  = 0.5;
p.sc50  = 0.05;   
p.ka    = 0.1;
p.ke    = 0.5;
p.V     = 20;
p.F     = 0.6;
p.dosetime = 0.01; % or 10:10:60;
p.dose  = 8*(ones(1,length(p.dosetime)));
p.route = 'IV';    % 'IV' or 'SC'
p.HbM = 140;
p.HbMax  = p.HbM;
p.HbMin  = p.HbM;
figure;
% prepare options for the DDE model
options = ddeset('RelTol',1e-5,'AbsTol',1e-5,'Events',@fevents);
sol = dde23(@derivatives,p.LS,[p.hb00 0 0],[0 p.tstop],options,p);
while sol.x(end) < p.tstop
    if length(sol.ie) > 1 && sol.ie(end-1)==8
        sol.ie
    end
    if sol.ie(end)<=length(p.dosetime)
       dd = p.dose(sol.ie(end));
    end
    route = strcmp(p.route,'SC');
    options = ddeset('RelTol',1e-7,'AbsTol',1e-7, ... 
        'InitialY',[sol.y(1,end) sol.y(2,end) + dd*route*p.F ...
        sol.y(3,end) + dd*(1-route)],'Events',@fevents);
    sol = dde23(@derivatives, p.LS,sol, [sol.x(end) p.tstop], ...
        options, p);
    subplot(3,1,1)  
    h311Hb = plot(sol.x, sol.y(1,:)','b','LineWidth',2);
    hold on
    h311Max = plot([0,p.tstop],[p.HbMax,p.HbMax],'b');
    h311Min = plot([0,p.tstop],[p.HbMin,p.HbMin],'b'); %#ok<NASGU>
    axis([0 p.tstop 100 150])
    ylabel('Hemoglobin [g/L]')
    
    subplot(3,1,2)   
    yy = prod(sol.x,sol.y(3,:),p); 
    h312Pr = plot(sol.x', yy,'g');
    hold on
    tt=p.LS+sol.x;
    index = tt <= p.tstop;
    h312R = plot(tt(index), yy(index),'r'); %#ok<NASGU>
    ylabel('Hb Rates [g/d]')
    
    subplot(3,1,3)   
    h313SC50 = plot([0 p.tstop],[p.sc50 p.sc50],'--b');
    xlabel('Time [d]')
    hold on
    if strcmp(p.route,'SC')
        [AX,h313DC,h313DA]=plotyy(sol.x, 1/p.V*sol.y(3,:), ...
            sol.x, 1/p.V*sol.y(2,:));
        set(h313DC, 'LineWidth',2)
        set(get(AX(1),'Ylabel'),'String', ... 
            'Drug Concentration [ng/mL]')
        set(get(AX(2),'Ylabel'),'String', 'Drug Amount [mg]')
    else
        h313DC = plot(sol.x, 1/p.V*sol.y(3,:),'LineWidth',2);  
        ylabel('Drug Concentration [ng/mL]')  
    end 
end

subplot(3,1,2)
h312El = plot([0 p.LS ],[p.s0 p.s0 ],'r');
legend([h312Pr h312El], 'Hb production','Hb elimination')
legend([h311Hb h311Max], 'Hb','Hb target range') 
if strcmp(p.route,'SC') 
    legend([h313SC50, h313DC, h313DA],'SC50 concentration', ...
        'Drug concentration', 'Drug amount SC')
else
 	legend([h313SC50, h313DC],'SC50 concentration', ... 
        'Drug concentration')
end
print('-dtiff','-r800',['epodosing_',p.route, ... 
     num2str(length(p.dosetime))])
end   


function dydt = derivatives(t, y, Z, p)
%DERIVATIVES Compute the right-hand side of the DDE.
%   DYDT = DERIVATIVES(T, Y, Z, P) calculates |DYDT|, he right-hand
%   side of the DDE model, at points defined by the vector |Y|, |T|,
%   and with parameters |P|. Matrix |Z| corresponds to all possible
%   delays and contains solutions |Y| at the delayed argument. Thus,
%   |Z| has the number of rows equal to the length of |Y|, and the
%   number of column equal to the number of delays. For example:
%   Z(3,1) means y(3) at time |T|-lag; lag is a delay.

tlag = t-p.LS;
m0   = y(2);
dm0  = -p.ka*m0;
m    = y(3);
dm   = p.ka*m0-p.ke*m;
c    = m/p.V;
mlag = Z(3,1);
clag = mlag/p.V;
rin  = prod(t,   c,   p);
rout = prod(tlag,clag,p);
dydt = [rin - rout ; dm0; dm];

end


function prod = prod(t, c, p)
%PROD evaluate Hb production
%   PROD = PROD(T,C,P) computes the hemoglobin production based on  
%   the drug effect formula (Hill equation). Input parameters:   
%   |T| - time, |C| - concentration, |P| - additional parameters 
%   (required by the Hill equation). 

E = hillEffect(c, 1, p.smax.*(t>=0), p.sc50, 1);
prod = p.s0.*E;

end


function E = hillEffect(c, E0, Emax, EC50, n)
%HILLEFFECT Compute drug effect based on the Hill equation.
%   E = HILLEFFECT(C, E0, EMAX, EC50, N) computes drug effect |E|, 
%   based on the Hill equation as a function of concentration |C|,
%   and with the following concentration-response parameters:
%   |E0| - baseline response, |EMAX| - maximum effect, 
%   |EC50| - concentration related to 50% of max. effect, 
%   and |N| - Hill coefficient of sigmoidicity.

E = E0 + Emax.*c.^n./(EC50.^n+c.^n);

end


function [value,isterminal,direction] = fevents(t, y, ~, p)
%FEVENTS Specify events for the DDE model.
%   [VALUE,ISTERMINAL,DIRECTION] = FEVENTS(T,Y,Z,P) detects the exact
%   time points of events specified as a function of time |T|, vector
%   of dependent variables |Y|, and a structure of some (specific)
%   other parameters |P|. Matrix |Z| corresponds to all possible
%   delays and contains solutions |Y| at the delayed argument.
%
%   Output parameters: |VALUE| - value investigated as event;
%   |ISTERMINAL| = 0|1 where 0 stands for 'don't stop', 1 for 'stop
%   integration'; |DIRECTION| = -1|0|1 where (-1) means 'negative
%   direction only', 0 - 'both direction', 1 - 'positive direction
%   only'

% define events: 
%   - dose times (time event)
%   - Hb level for dose changes

value(:,1) = [t - p.dosetime'; y(1)-p.HbM]; 
isterminal(:,1) = [ones(1,length(p.dosetime))'; 0];
direction(:,1)  = [ones(1,length(p.dosetime))'; 1];

end


