function esaCTSextend(regimenType, numSub, runs)
%ESACTS Launch the Clinical Trial Simulation (CTS) for ESA model.
%   ESACTS(REGIMENTYPE,NUMSUB,RUNS) initializes and controls the 
%   CTS based on the PK-PD ESA model. The simulation is performed 
%   simultaneously for |NUMSUB| subjects, and consists of |RUNS|  
%   repetitions. Graphs with plasma concentration, hemoglobin time   
%   courses, and dose adaptation over time are initialized for the  
%   simulation time. |REGIMENTYPE| is linked to the dose regimen and
%   must be here protocol' but the applied dose schedule function  
%   can be easily extended for other regimens.
%
%   Example for calling:   esaCTS('protocol', 2, 10)
%   This will simulate the drug PK-PD 2 times, each time with 10 
%   subjects 
%
%   (see also esaCTS>DOSESCHEDULE). 

%   Check input arguments and provide default values if necessary
error(nargchk(0,3,nargin)); %#ok<NCHKN>
if nargin < 3
    runs = 10;
end
if nargin < 2
    numSub = 2;
end
if nargin < 1
    regimenType = 'protocol';
end

% create a data file for statistics (results database). To avoid 
% that the file will be overwritten, the file name is linked to the
% current date and time.
c = fix(clock);
dt = [num2str(c(1)) num2str(c(2)) num2str(c(3)) '_'  ... 
      num2str(c(4)) num2str(c(5)) num2str(c(6))];
filename = [dt '.STA'];
fclose(fopen(filename, 'w')); 
for nr=1:runs
    figs = createFigs(1);
    esaCTSmain(regimenType, numSub, nr, figs, filename);
    if nr == 1      % save only figures from the first replication 
        print(figs.consFigure, '-r300', '-dtiff', ['cons', '_'])
        print(figs.HbFigure, '-r300', '-dtiff', ['Hb', '_'])   
        print(figs.doseFigure, '-r300', '-dtiff', ['dose', '_'])
        close([figs.consFigure, figs.HbFigure, figs.doseFigure])
    else
        close([figs.consFigure, figs.HbFigure, figs.doseFigure])
    end;      
end

end


function esaCTSmain(regimenType, numSub, numRun, graph, filename)
%ESACTSMAIN Conduct a trial simulation for the ESA model. 
%   ESACTSMAIN(REGIMENTYPE,NUMSUB,NUMRUN,GRAPH,FILENAME) conducts  
%   one trial simulation for |NUMSUB| subjects simultaneously.
%   |NUMRUN| denotes the current repetition number, |REGIMENTYPE| -
%   dose regimen and |FILENAME| - a file for saving the simulation
%   results. The structure |GRAPH| contains handles for figure   
%   windows and axes used for graphical presentation. 
%   See also esaCTS.

% Reset the random number generator to the default
if numRun == 1 
     rng default 
end
% Calculate dosing times and amount based on regimen
p = initializeParams(numSub);
[doseTimes, ~] = doseSchedule(regimenType);
% Correlate ESA2 dose and ES1 dose
doseAmount0 = p.doseESAcov;

consAxes = graph.consAxes;
HbAxes = graph.HbAxes;
doseAxes = graph.doseAxes;

% Simulate system for all subjects simultaneously
    
% Initialize parameters and set initial values for all subjects
% p = initializeParams(numSub);
% complete structure p (used in several functions)
p.numSub = numSub;
p.doseAmount(1:numSub) = doseAmount0;  
p.doseTimes = doseTimes;
p.doseTrue(1:numSub) = 1;
p.lastAdapt(1:numSub) = -28;
p.switcher = 1;
p.tmax = doseTimes(end);
p.inity = zeros(1, numSub);
p.numRun = numRun;
p.doseNext = 2;

sol = [p.doseAmount p.c0 p.Hbsw]';  
delay = p.LS;

% Simulate system for each treatment period
options = ddeset('Events',@fevents, 'RelTol',1e-5, 'AbsTol', 1e-5);
% Set time interval for this treatment period
tspan = [doseTimes(1), doseTimes(end)]; 

t0 = doseTimes(1);
tx = t0;
ie = numSub + 1;
p.dosex = p.doseAmount;
while t0 < p.tmax 
    if ie > numSub  
        % decrease accuracy to detect the next event 
        % (after a 'dose time' event)
        options.AbsTol = 1e-4;
        options.RelTol = 1e-4;
        showDosing(tx, t0, p.dosex, p.doseAmount, numSub, doseAxes)
        % Save the current dose and time
        tx = t0;
        p.dosex = p.doseAmount;
    else
        % increase accuracy to detect the next event 
        % (after an 'end life span' event)
        options.AbsTol = 1e-6;
        options.RelTol = 1e-6;
    end
    sol = dde23(@derivatives, delay, sol, tspan, options, p);  
    te = sol.xe'; 
    %ye = sol.ye'; 
    ie = sol.ie;
    % Plotting time courses 
    tPoints  = sol.x; 
    concPoints = sol.y(  numSub+1 : 2*numSub, :); 
    HbPoints   = sol.y(2*numSub+1 : 3*numSub, :);       
    plot(consAxes, tPoints, concPoints)
    plot(HbAxes, tPoints, HbPoints) 
    drawnow
    ie = ie(te == te(end));     % eliminate already handled events
    p.te = te(end);
    p = todoevents(sol.y', ie, p);
    % Prepare next period (next dose)
    t0 = sol.x(end);
    tspan = [t0 p.tmax]; 
    sol.ie = [];
    sol.xe = [];
    sol.ye = [];
    sol.discont = [];
    sol.y(1:numSub, end) = p.inity;
end 
% save data for the statistical test (Hb end values)
savedata(filename, numRun, sol.y(:, end))

end


function [doseTimes, doseAmount] = doseSchedule(regimenType)
%DOSESCHEDULE Create a vector of dosing times and amounts of drug.
%   [DOSETIMES, DOSEAMOUNT] = DOSESCHEDULE(REGIMENTYPE) returns 
%   a vector |DOSETIMES| of dosing times and a scalar |DOSEAMOUNT| 
%   of dosing amount. Currently, |REGIMENTYPE| can only be  
%   'protocol' but there placeholder for other options.


treatmentWeeks = 56;
cycleWeeks     = 2;
daysInWeek     = 7;
hoursInDay     = 24;
initialTime    = 0;

% treatment time [hours]
endTime = treatmentWeeks * daysInWeek * hoursInDay;
% cycle time [hours]
cycleTime = cycleWeeks * daysInWeek * hoursInDay;

switch regimenType
    case 'protocol' 
        dailyDoses = 1;      
        doseInitial = 20;   % dose in [ng]
        timeOnDrug = 0;     % (first dose time) = (last dose time)
    % case 'other'  % placeholder for other regimen(s)
    otherwise
        messageID = 'Book:esaCTS:UnknownRegimen';
        messageStr = ['Unknown regimen ''%s''. Regimen must be ',...
            '''protocol''.'];
        error(messageID, messageStr, regimenType);
end 

% create vector of treatment times
initialCycleTimes = ... 
    initialTime : cycleTime : (endTime - cycleTime);
doseTimesWithinCycle = ...
    initialTime : hoursInDay/dailyDoses : timeOnDrug; 
doseTimes = reshape(...
    repmat(doseTimesWithinCycle', 1, length(initialCycleTimes))+ ...
    repmat(initialCycleTimes, length(doseTimesWithinCycle), 1), ...
    1, length(initialCycleTimes)*length(doseTimesWithinCycle));
doseTimes = [doseTimes, endTime]; % adding end time of treatment
doseTimes = doseTimes/24;
doseAmount =  doseInitial; 

end


function params = initializeParams(numSub)
%INITIALIZEPARAMS Create initial values for model parameters.
%   PARAMS = INITIALIZEPARAMS(NUMSUB) returns a structure 
%   |PARAMS| containing initial values for model parameters, 
%   including variability when necessary. The values are created
%   for all simulated subjects; number of subjects is |NUMSUB|. 

% Scope of variability 
params = [];
pkCV = 0.32;
% PK parameters 
ka = 0.825;                     % 1-st order absorption rate [1/day]
kaCV = 0.82;             
params.ka = variability('varnorm', ka, ka*kaCV, numSub, 0.5);
F = 0.394;                      % bioavailability 
params.F = F*ones(1,numSub);    % no variability
CL = 0.749;                     % drug clearances [L/day]
CLv = variability('varnorm', CL, CL*pkCV, numSub, 0.3); 
% weight covariate effect on CL
bwref = 66;         % reference value for the body weight covariate
bwCL = 0.571;   
bwcov = covData(75, 10, numSub, 50, 90, 'normal');  % body weight
params.CL = CLv.*(bwcov/bwref).^bwCL;
V  = 4.72;                      % volumes of distribution [L]
Vv = variability('varnorm', V, V*pkCV, numSub, 0.3);
% weight covariate effect on V
bwref = 66;         % reference value for body weight covariate  
bwV = 0.443;   
% age covariate effect on V
aref = 54;          % reference value for the age covariate
aV = 0.443;   
acov = covData(75, 10, numSub, 20, 90, 'normal');
params.V = Vv.*(bwcov/bwref).^bwV.*(acov/aref).^aV;
params.c0 = zeros(1,numSub);    % initial concentrations 
% PD parameters 
LS = 61.3;                      % life span of RBC [day]  
LSCV = 0.32;
params.LS = variability('varnorm', LS, LS*LSCV, numSub,30); 
Hb0 = 9.3;                      % Hb conc. at baseline [ng/mL]
Hb0CV = 0.25;  
H1 = covData(Hb0, Hb0*Hb0CV, numSub, 7, 10.5, 'normal');
H2 = covData(Hb0, Hb0*Hb0CV, numSub, 10.5, 12.5, 'normal');
params.Hb0 = min(H1,H2);      
params.Hbsw = max(H1,H2);       % Hb conc. at switch   [ng/mL]
Smax = 0.425;                   % max effect 
SmaxCV = 1.42;
params.Smax = variability('varnorm', Smax, Smax*SmaxCV, numSub, 0.5);
SC50 = 0.898;       % concentration linked to 50% of max effect 
SC50CV = 5.59;  
SC50v = variability('varnorm', SC50, SC50*SC50CV, numSub, SC50); 
% ESA dose covariate effect on SC50
doseESAref = 10^2.5;         
doseESASC50 =  0.3;         
a = 1; b = 4;
doseESAcov = rand(1, numSub)*(b - a) + a;
params.doseESAcov = 10.^doseESAcov; 
params.SC50 = SC50v.*(params.doseESAcov./doseESAref).^doseESASC50;

end


function x = variability(distribution, m, s, num, lim) 
%VARIABILITY Generate variability.
%   X = VARIABILITY(DISTRIBUTION, M, S, NUM, LIM) returns a vector 
%   of random numbers for variability, specified by |DISTRIBUTION|   
%   with the mean value |M| and standard deviation |S|. |NUM| is the
%   length of the vector |X|.
%   |DISTRIBUTION| can be 'varlog' or 'varnorm':  
%   'varlog' generates random numbers |X| ~ logN(|M|,|S|).
%   |M| stands for mean value of the lognormal distribution 
%   |S| - standard deviation of the lognormal distribution
%   'varnorm' generates normally distributed random numbers |X|,  
%   where |X| ~ N(|M|,|S|), right-censored by |LIM|. 

switch distribution
    case 'varlog'
        % lognormal distribution
        mu    = log(m^2/sqrt(m^2 + s^2));   % where: N(|mu|,|sigma|)
        sigma = sqrt(log(1 + (s/m)^2));     % where: N(|mu|,|sigma|) 
        x     = lognrnd(mu,sigma, 1, num);
    case 'varnorm'
        % normal distribution
        mu    = m;
        sigma = s;
        x     = max(normrnd(mu, sigma, 1, num), lim);
    otherwise 
        % placeholder for other distributions
end

end


function cx = covData(varargin)
%COVADATA Generate demographics.
%   CX = COVADATA(VARARGIN) generates an 1-by-n array with random
%   demographic data, for incorporation of a covariate effect in the
%   trial simulation. Calling types:
%   CX = COVADATA(MU,SIGMA,N,CXMIN,CXMAX,TYPE) for generation of |N|
%   normally distributed values, with the mean |MU| and standard
%   deviation |SIGMA| that represent such covariates as body weights,
%   heights, etc.; |TYPE| must be 'normal'. The values are truncated
%   to an interval with lower cutoff |MIN| and upper cut off |MAX|.
%   CX = COVADATA(N,TYPE) for binary distributed data that represent
%   covariates like sex, smokers ('yes','no'), etc. |TYPE| must be
%   'binary'.

na = nargin; 
switch na 
    case 6 
        % normal distribution
        mu = varargin{1};       % mean      
        sigma = varargin{2};    % standard deviation 
        n = varargin{3};        % length of the data array 
        cxmin = varargin{4};    % 
        cxmax = varargin{5};
        type = varargin{6};
        if ~strcmp(type, 'normal'); errormessage(1); end
        cx(1:n) = 0;
        for i = 1:n
            x = normrnd(mu,sigma,1,1);
            in = cxmin <= x & x <= cxmax;    
            while ~in 
               x = normrnd(mu, sigma, 1, 1);
               in = cxmin <= x & x <= cxmax; 
            end
            cx(i) = x;
        end
    case 2
        % binary distribution
        n = varargin{1};
        type = varargin{2}; 
        if ~strcmp(type, 'binary'); errormessage(1); end
        cx = binornd(1,0.5,1,n);
    otherwise 
        errormessage(3) 
end  
       
    function errormessage(em)
    %ERRORMESSAGE Error message. 
    %   ERRORMESSAGE(EM) displays an error message if the arguments, 
    %   varargin, passed into covarEffect are not correct. The 
    %   parameter |EM| is a message identifier number.    
    %   The function is nested due to |type| and |na|.
    
    switch em
        case 1
            messageID = 'Book:covarEffect:type';
            messageStr = 'Verify the distribution type ''%s''';
            error(messageID, messageStr,type); 
        case 2
            messageID = 'Book:covarEffect:type';
            messageStr = 'Verify the distribution type ''%s''';
            error(messageID, messageStr,type);
        case 3
            messageID = 'Book:covarEffect:nargin';
            messageStr = 'Verify the number of arguments ''%d''';
            error(messageID, messageStr,na);                
    end

    end
end


function E = hillEffect(c, E0, Emax, EC50, n)
%HILLEFFECT Compute drug effect based on the Hill equation.
%   E = HILLEFFECT(C, E0, EMAX, EC50, N) computes drug effect |E|, 
%   based on the Hill equation as a function of concentration |C|,
%   and with the following concentration-response parameters:
%   |E0| - baseline response, |EMAX| - maximum effect, 
%   |EC50| - concentration related to 50% of max. effect, 
%   and |N| - Hill coefficient of sigmoidicity.

E = E0 + Emax.*c.^n./(EC50+c.^n); 

end


function dydt = derivatives(~, y, Z, p)
%DERIVATIVES Compute the right-hand side of the DDE.
%   DYDT = DERIVATIVES(T, Y, Z, P) calculates |DYDT|, he right-hand
%   side of the DDE model, at points defined by the vector |Y|, |T|,
%   and with parameters |P|. Matrix |Z| corresponds to all possible
%   delays and contains solutions |Y| at the delayed argument. Thus,
%   |Z| has the number of rows equal to the length of |Y|, and the
%   number of column equal to the number of delays. For example:
%   Z(2,1) means y(2) at time |T|-clag; clag is a delay.

ns = p.numSub;
% set dependent variables for all subjects simultaneously
% (vectorization)
a  = y(     1 :   ns)';     % drug amount                                
c  = y(  ns+1 : 2*ns)';     % concentration
Hb = y(2*ns+1 : 3*ns)';     %#ok<NASGU> % hemoglobin  
cdelay = diag(Z(ns+1 : 2*ns, 1:ns))'; % delay matrix linked to |c|
                                   
% PK Model (SC administration)
dAdt = -p.ka.*a;
dcdt  = (p.ka./p.V./p.F.*a - p.CL./p.V.*c);  

% PD Model
SESA = (p.Hb0 - p.Hbsw)./p.LS .* p.switcher;
EC      = hillEffect(c, 0, p.Smax, p.SC50, 1);
ECdelay = hillEffect(cdelay, 0, p.Smax, p.SC50, 1);
S       = p.Hb0./p.LS.*(1 + EC); 
Sdelay = p.Hb0./p.LS.*(1 + ECdelay); 
dHbdt  = S - Sdelay + SESA; 
% Derivatives vector of ODE system
dydt = [ dAdt'        
         dcdt'          
         dHbdt'];  
end


function [value,isterminal,direction] = fevents(t, ~ , ~, p)
%FEVENTS Specify events for the DDE model.
%   [VALUE,ISTERMINAL,DIRECTION] = FEVENTS(T,Y,Z,P) detects the exact
%   time points of events specified as a function of time |T|, vector
%   of dependent variables |Y|, and a structure of some (specific)
%   other parameters |P|. Matrix |Z| corresponds to all possible
%   delays and contains solutions |Y| at the delayed argument.
%
%   Output parameters: |VALUE| - value investigated as event;
%   |ISTERMINAL| = 0|1 where 0 stands for 'don't stop', 1 for 'stop
%   integration'; |DIRECTION| = -1|0|1 where (-1) means 'negative
%   direction only', 0 - 'both direction', 1 - 'positive direction
%   only'
   
ns = p.numSub;

% Event: end of life-span (LS)
direction1(1:ns,1) = 1;
isterminal1(1:ns,1) = 1;
value1(1:ns,1) = t - p.LS;

% Event: Dosing
direction2(1,1) = 1;
isterminal2(1,1) = 1;
value2(1,1) = t - p.doseTimes(p.doseNext);

% Event: Interrupt the proces after 180 days            %EXT
direction3(1,1) = 1;                                    %EXT
isterminal3(1,1) = 1;                                   %EXT
value3(1,1) = t - 168;                                  %EXT

% Event 1 + 2 + ...
direction = [direction1; direction2; direction3];       %EXT
isterminal = [isterminal1; isterminal2; isterminal3];   %EXT                     
value = [value1; value2; value3];                       %EXT 

end


function pMod  = todoevents(y, numEvents, p)
%TODOEVENTS Handle detected events.
%   PMOD = TODOEVENTS(Y, NUMEVENTS, P) conducts a proper action if
%   events happen. The function handles events detected at 1 time
%   point. In typical cases 1 event is linked to 1 time point.
%   However, in general it may happen that simultaneous events take
%   place, and the function then handles more than 1 event detected
%   at the same time. |NUMEVENTS| is a vector with event numbers, 
%   |P| - structure with parameters. |Y| - current solution values
%   (dependent variables). Output parameters: |PMOD| is a changed
%   |P|, according to action undertaken.

pMod = p;
ns = pMod.numSub;
% use 'for' as more than 1 event can happen at the same time
for i = numEvents
    % identify subject number
    switch i
        case num2cell(1:ns)
            % end of life-span (LS)
            subject = (p.numRun-1)*ns + i;
            display(['time:', num2str(p.te), ... 
                ', event:end of life-span, ', 'subject:', ...  
                num2str(subject)])  
            pMod.switcher = 0;
            pMod.inity(1:ns) = y(end, 1:ns);
        case num2cell(ns+1)
            % dosing time
            display(['time:', num2str(p.te), ', event:dosing time'])                   
            [pMod.doseTrue, pMod.lastAdapt] = doseAdaptation(p, y);
            pMod.doseAmount = pMod.doseAmount.*pMod.doseTrue; 
            pMod.doseAmount(pMod.doseAmount == 0 & ...
                pMod.doseTrue > 3/4) = 1; 
            pMod.inity(1:ns) = y(end, 1:ns) + pMod.doseAmount; 
            pMod.doseNext = pMod.doseNext + 1;  
        case num2cell(ns+2)                                   %EXT
            % dosing time                                     %EXT
            display(['time:', num2str(p.te), ...              %EXT
                ', event:INTERRUPTION !!! (%EXT)'])           %EXT 
            if input('Stop the process ? (y/n)', 's') == 'y'  %EXT
                pMod.tmax = 168;                              %EXT
            end                                               %EXT
        otherwise
            messageID = 'Book:CTS:UnknownEvent';
            messageStr = 'Unknown event ''%s''.';
            error(messageID, messageStr, num2str(i));
    end
end

end


function [doseAdapt, lastAdapt] = doseAdaptation(p, y) 
%DOSEADAPTATION Perform dose adaptation.
%   [DOSEADAPT LASTADAPT] = DOSEADAPTATION(P, Y) specifies dose 
%   changes in terms of absolute Hb values, Hb values relative 
%   to baseline, and Hb values observed within any 4-weeks period. 
%   Dose changes could start at each dosing time (1x/2 weeks), but 
%   once the first dose adaptation has been performed the next dose 
%   adaptations can't occur more often than 1x/4 weeks. |Y| stands
%   for independent variables in the model, and |P| - a structure 
%   with parameters needed for validation of the adaptation rules.
%   This function returns |DOSEADAPT|- vector with values showing  
%   how the doses were changed, and |LASTADAPT| - vector with times 
%   when the doses were changed; both regarding all subjects.

adaptRule = (p.doseTimes(p.doseNext) - p.lastAdapt) >= 28; % 4-weeks
doseAdapt = p.doseTrue; 
Hb = y(end, 2*p.numSub+1 : 3*p.numSub);
% Hemoglobin difference from baseline - levels
deltaHb = Hb - p.Hb0;
a =                 deltaHb < -2;
b = -2 <= deltaHb & deltaHb < -1; 
c = -1 <= deltaHb & deltaHb <  1;
d =  1 <= deltaHb & deltaHb <  2;
e =  2 <= deltaHb ;
% dose adaption to hemoglobin concentration - levels
doseAdapt(             Hb <   8      ) = 2;
doseAdapt( 8   <= Hb & Hb <   9      ) = 3/2;
doseAdapt( 9   <= Hb & Hb <  10.5    ) = 5/4;
doseAdapt(10.5 <= Hb & Hb <= 13.5 & a) = 3/2;
doseAdapt(10.5 <= Hb & Hb <= 13.5 & b) = 5/4;
doseAdapt(10.5 <= Hb & Hb <= 13.5 & c) = 1;
doseAdapt(10.5 <= Hb & Hb <= 13.5 & d) = 3/4;
doseAdapt(10.5 <= Hb & Hb <= 13.5 & e) = 1/2;
doseAdapt(13.5 <  Hb & Hb <= 14      ) = 3/4; 
doseAdapt(14   <  Hb                 ) = 0;
% save adaptation flag and adaptation time for each subject 
doseAdapt = doseAdapt.*adaptRule + ~adaptRule;
lastAdapt = p.doseTimes(p.doseNext)*adaptRule + ... 
    p.lastAdapt.*(doseAdapt == 1);
end


function fg = createFigs(selgraph)
%CREATEFIGS Prepare figures.
%   FG = CREATEFIGS(SELGRAPH) creates figure objects, indicated by  
%   the vector |SELGRAPH|, for several graphs used during the trial 
%   simulation. The handles of created objects (figure windows and 
%   axes) are stored in the structure |FG|. 

% 1 - Set up figures for time courses 
if ismember(1, selgraph)
    % Concentration
    fg.consFigure = figure;
    fg.consAxes = axes;
    set(fg.consAxes, 'FontSize', 15, 'YScale', 'log')
    xlabel('Time [d]')
    ylabel('Concentration [ng/mL]')
    title('Concentration Time Course')
    hold on; grid on;

    % hemoglobin time courses 
    fg.HbFigure = figure;
    fg.HbAxes = axes;
    set(fg.HbAxes, 'FontSize', 15)
    xlabel('Time [d]')
    ylabel('Hb [g/dL]')
    title('Hb Time Course')
    hold on; grid on;
    
    % dosing
    fg.doseFigure = figure;
    fg.doseAxes = axes;
    set(fg.doseAxes, 'FontSize', 15, 'YScale', 'log')
    xlabel('Time [d]')
    ylabel('Dose [ng]')
    title('Dose Adaptation')    
%     title('No Dose Adaptation')
    hold on; grid on;
    set([fg.consFigure; fg.HbFigure; fg.doseFigure], 'units', ...  
       'normalized', {'Position'}, {[0.0, 0.4, 0.3, 0.4]; ... 
       [0.35, 0.4, 0.3, 0.4]; [0.7, 0.4, 0.3, 0.4]});    
end

% 2 - Set up a figure for CTS power values
if ismember(2, selgraph)
    fg.powerFigure = figure;
    fg.powerAxes = axes;  
    set(fg.powerAxes, 'FontSize', 15, 'XScale', 'log')
    xlabel(' ')
    set(fg.powerFigure, 'units', 'normalized', 'Position', ... 
        [0.1, 0.1, 0.6, 0.15], 'PaperPositionMode','auto')
    set(fg.powerAxes, 'YTickLabel',['';'';''],'YTick',[],'Box','on')
    hold on
end

end


function showDosing(t1, t2, val1, val2, ns, haxes)
%SHOWDOSING Show current and previos dose values.
%   SHOWDOSING(T1, T2, VAL1, VAL1, NS, HAXES) plots values of two
%   vectors, |VAL1| and |VAL2|, each containing |NS| elements, at
%   time points |T1| and |T2|. The graph axes are pointed by |HAXES|.
%
%   This function if used to show the difference between current and
%   previous dose.

% use 'xr' if jitter is needed  to visualize multiple values 
xr = rand(1,ns); %#ok<NASGU
plot(haxes, [t1, t2]', [val1; val2], '-+')

end


function savedata(filename, n, y)
%SAVEDATA Save simulation results.
%   SAVEDATA(FILENAME,N,Y) saves in the text file |FILENAME| final
%   values of the hemoglobin level, obtained in the repetition |N|.
%   The hemoglobin values are stored in the ODE model solution |Y|.
 
fid = fopen(filename, 'a');
lc = length(y);        % number of columns in the solution
nHb =  lc/3*2+1 : lc;  % select only Hb columns 
fprintf(fid, '%s \n', num2str([n y(nHb)']));
fclose(fid);

end


