function esaCTSpower
%ESACTSPOWER Calculate the power of a trial simulation
%   ESACTSPOWER reads from external files the results of two ESA
%   trial simulations and calculates the power.  
%
%   See also esaCTS.
  
fileNoAdapt  = '2013519_141618_000.STA'; 
fileAdaptDip = '2013519_17927_DIP.STA';
fileAdaptInd = '2013519_194336_IND.STA';
% find the number of lines = the number of replications
fid = fopen(fileNoAdapt, 'r');   
nLines = 0; 
while (fgets(fid) ~= -1);  nLines = nLines+1;  end
reps= nLines;
alpha = 0.05;

% trial simulation with 2 dependent samples  
p = analysis(fileNoAdapt, fileAdaptDip, reps, 'dependent');
figs = createFigs(2);
outcomes(p, figs.powerAxes, alpha); 
print(figs.powerFigure, '-dtiff','-r600','esaCTSpowerDIP')

% trial simulation with 2 independent samples
p = analysis(fileNoAdapt, fileAdaptInd, reps, 'independent');
figs = createFigs(2);
outcomes(p, figs.powerAxes, alpha); 
print(figs.powerFigure, '-dtiff','-r600','esaCTSpowerIND')

end


function pval = analysis(filename1, filename2, reps, testcase)
%ANALYSIS Verify effects of the active drug adaptation.
%   PVAL = ANALYSIS(FILENAME1, FILENAME1, REPS, TESTCASE) compares
%   two groups, the first treated without dose adaptation the
%   without. The results databases stored in files |FILENAME1| and
%   |FILENAME2| for the first and second group respectively (produced
%   in trial simulations by the esaCTS.m program). The statistical
%   z-test is used to verify the null-hypothesis H0 (there is no
%   statistically significant effect of the drug adaptation) against
%   the alternative hypothesis (the drug adaptation is significant).
%   The verification is carried out pairwise taking into account
%   results from the respective replication in both scenarios (with
%   and without adaptation), i.e. it is repeated |REPS| times. For
%   each replication pair, a p-value is computed, and finally a
%   vector |PVAL| is created that contains all p-values. The
%   comparison is possible for dependent samples: |TESTCASE| =
%   'dependent', or independent: |TESTCASE| = 'independent'.
%   See also esaCTS.

fid1 = fopen(filename1,'r');  X1 = fscanf(fid1,'%f');  fclose(fid1);
fid2 = fopen(filename2,'r');  X2 = fscanf(fid2,'%f');  fclose(fid2);
numSub = length(X1)/reps;       % or numSub = length(X2)/reps;
% Ignore first column
X1 =  reshape(X1, numSub, reps)'; X1r = X1(:,2:end);
X2 =  reshape(X2, numSub, reps)'; X2r = X2(:,2:end);  
%predifined range of Hb values
X1in = 10.5 <= X1r & X1r <= 12.5;        
X2in = 10.5 <= X2r & X2r <= 12.5;

switch testcase
    case 'dependent'
        % One-tailed McNemar test (one-tailed) 
        xYesYes = sum( X1in &  X2in, 2);  a = xYesYes; 
        xYesNo  = sum( X1in & ~X2in, 2);  b = xYesNo;         
        xNoYes  = sum(~X1in &  X2in, 2);  c = xNoYes;         
        xNoNo   = sum(~X1in & ~X2in, 2);  d = xNoNo;
        pval = zeros(numSub,1);
        for nr = 1:reps
            n = b(nr) + c(nr);
            mbc = min([b(nr), c(nr)]);
            pval(nr) = 0;
            for m = 0:mbc
                biC = factorial(n)/(factorial(m)*factorial(n-m)); 
                pval(nr) = pval(nr) + biC*(1/2)^n;  
            end
        end 
    case 'independent'
        % z test (if samples are independent)
        yNaYes = sum( X1in,2);  a = yNaYes; % no adapt. - 'success'  
        yNaNo  = sum(~X1in,2);  b = yNaNo;  % no adapt. - 'failure'
        yAdYes = sum( X2in,2);  c = yAdYes; %    adapt. - 'success' 
        yAdNo  = sum(~X2in,2);  d = yAdNo;  %    adapt. - 'failure'
        p1 = a./numSub; p2 = c./numSub; 
        p = (a+c)./(2*numSub); q = 1-p;
        z = (p1-p2)./sqrt(p.*q./numSub+p.*q./numSub);
        pval = normcdf(z);
    otherwise
end

end


function outcomes(p, haxes, alpha)
%OUTCOMES Show the results of the CTS power calculation.
%   OUTCOMES(P, HAXES, ALPHA) produces a graph in the axis box
%   indicated by the handle |HAXES|. It shows how p-values, |P|, 
%   are distributed against |ALPHA| (the significance level for
%   acceptance or rejection of the hypothesis H0).

power = sum(p < alpha)/length(p);
stem(haxes, p, ones(length(p),1))
hold on
stem(haxes, alpha, 1,'+r','LineWidth',2)
legend('p-value', ['{\alpha} = ', num2str(alpha)],'Location', ...
    'SouthWest')
title(['CTS: power = ', num2str(power)])
display(['CTS: power = ', num2str(power)])

end


function fg = createFigs(selgraph)
%CREATEFIGS Prepare figures.
%   FG = CREATEFIGS(SELGRAPH) creates figure objects, indicated by  
%   the vector |SELGRAPH|, for several graphs used during the trial 
%   simulation. The handles of created objects (figure windows and 
%   axes) are stored in the structure |FG|.  

% 1 - Set up figures for time courses 
if ismember(1, selgraph)
    % Concentration
    fg.consFigure = figure;  fg.consAxes = axes;
    set(fg.consAxes, 'FontSize', 15, 'YScale', 'log')
    xlabel('Time [d]');  ylabel('Concentration [ng/mL]')
    title('Concentration Time Course')
    hold on; grid on;

    % hemoglobin time courses 
    fg.HbFigure = figure;   fg.HbAxes = axes;
    set(fg.HbAxes, 'FontSize', 15)
    xlabel('Time [d]');   ylabel('Hb [g/dL]')
    title('Hb Time Course')
    hold on; grid on;
    
    % dosing
    fg.doseFigure = figure;
    fg.doseAxes = axes;
    set(fg.doseAxes, 'FontSize', 15, 'YScale', 'log')
    xlabel('Time [d]');  ylabel('Dose [ng]')
    title('Dose Adaptation')    
%     title('No Dose Adaptation')
    hold on; grid on;
    set([fg.consFigure; fg.HbFigure; fg.doseFigure], 'units', ...  
       'normalized', {'Position'}, {[0.0, 0.4, 0.3, 0.4]; ... 
       [0.35, 0.4, 0.3, 0.4]; [0.7, 0.4, 0.3, 0.4]});    
end

% 2 - Set up a figure for CTS power values
if ismember(2, selgraph)
    fg.powerFigure = figure;
    fg.powerAxes = axes;  
    set(fg.powerAxes, 'FontSize', 15, 'XScale', 'log')
    xlabel(' ')
    set(fg.powerFigure, 'units', 'normalized', 'Position', ... 
        [0.1, 0.1, 0.6, 0.2], 'PaperPositionMode','auto')
    set(fg.powerAxes, 'YTickLabel',['';'';''],'YTick',[],'Box','on')
    hold on
end

end

