function evMultiProblem  
%EVMULTIPROBLEM Event handling in a two-compartment ODE model   
%   EVMULTIPROBLEM implements state events in a two-compartment 
%   model. Plasma concentration should be kept within a given range
%   by infusion rate: if concentration achieves allowed maximum then 
%   decrease infusion; if minimum - infusion has to be increased.  

% PK parameters
CL = 1.38E+01; V1 = 1.48E+01; Q = 2.17E+00; V2 = 4.23E+00; R = 50;         
% p structure with PK parameters available in several functions 
p.R0 = R; p.R = R; p.V1 = V1; p.V2 = V2; p.k = CL/V1;            
p.k12 = Q/V1; p.k21 = Q/V2;  
options = odeset('Events',@fevents, 'RelTol',1e-9, 'AbsTol', 1e-9);
tmax = 30; tspan = [0 tmax]; t = 0; c0 = [0 0];   % Initial values
while t(end) < tmax
    [t, c, te, ye, ie] = ode45(@derivatives, tspan, c0, options, p); 
    plot(t, c, 'LineWidth', 2); hold on  
    if isempty(ie) 
        break; 
    end
    ie = ie(te == te(end)); % eliminate already handled events 
    tspan = [te(end); tmax]; 
    c0 = [ye(end,1) ye(end,2)];
    display(['time: ', num2str(te(end)), '  events: ', ... 
        num2str(ie')])
    p = todoevents(ie', p);
end
xlabel('Time [hours]'); ylabel('Concentration [mg/L]');
legend('central compartment','peripheral compartment')
title('Two-Compartment Model / Infusion / Multiple Doses')  

end


function dcdt = derivatives(~, c, p)
%DERIVATIVES Compute the right-hand side of the ODE.
%   DYDT = DERIVATIVES(T, Y, P) calculates |DYDT|, the right-hand
%   side of the ODE model, at points defined by the vector of
%   dependent variables |Y|, time |T|, and with parameters |P|.

dcdt = [ p.R/p.V1 - (p.k + p.k12)*c(1) + p.k21*p.V2/p.V1*c(2)    
         p.k12*p.V1/p.V2*c(1) - p.k21*c(2)                    ];      
end


function [value,isterminal,direction] = fevents(t,y,~)
%FEVENTS Specify events for the ODE model.
%   [VALUE,ISTERMINAL,DIRECTION] = FEVENTS(T,Y,P) detects the exact
%   time points of events specified as a function of time |T|,
%   vector of dependent variables |Y|, and a structure of some )
%   (specific) other parameters |P|.

% Event 1
isterminal(1) = 1;      % Stop integration
direction(1) = 1;       % Positive direction only
value(1) = (t - 3);     % after 3 hours
% Event 2: Concentration at lower limit
isterminal(2) = 1;      % Stop integration
direction(2) = -1;      % Negative direction only
value(2) = (y(1) - 0.5); 
% Event 3: Concentration at upper limit
isterminal(3) = 1;      % Stop integration
direction(3) = 1;       % Positive direction only
value(3) = (y(1) - 2);  
% Event 4: test event forced to happen together with event 4 
isterminal(4) = 1;      % Stop integration
direction(4) =  1;      % Negative direction only
value(4) = t - 14.459115546074692;

end


function pMod = todoevents(numEvents, p)
%TODOEVENTS Handle detected events.
%   PMOD = TODOEVENTS(Y, NUMEVENTS, P) conducts a proper action if
%   events happen. The function handles events detected at 1 time
%   point. In typical cases 1 event is linked to 1 time point.
%   However, in general it may happen that simultaneous events take
%   place, and the function then handles more than 1 event detected
%   at the same time. |NUMEVENTS| is a vector with event numbers,
%   |P| - structure with parameters. |Y| - current solution values
%   (dependent variables). Output parameters: |PMOD| is a changed
%   |P|, according to action undertaken.

pMod = p;
% use 'for' as more than 1 event can happen at the same time
for i = numEvents
    switch i
        case 1
            % stop infusion after 3 hours 
            display('event 1: handled')
            pMod.R = 0; 
        case 2
            %  allow full infusion
            display('event 2: handled')                
            pMod.R = p.R0;            

        case 3
            %  reduce infusion  
            display('event 3: handled')
            pMod.R = p.R0/10;     
        case 4
            % show a message to event 4
            display('-----------------------------')
            display('event 4 - TEST EVENT: handled')
            display('-----------------------------')   
        otherwise
            messageID = 'Book:infus2comstev:UnknownEvent';
            messageStr = 'Unknown event ''%s''.';
            error(messageID, messageStr, num2str(i));
    end
end

end
