function hoghux()
%HOGHUX simulate the action potential in a single neuron.
%   HOGHUX()simulates a single neuron under deterministic stimulation
%   using the full Hodgkin-Huxley model (in 2 dimensional domain,
%   i.e. time and distance). The model is described by a system of
%   partial differential equations. To solve them, the compartmental
%   method for second derivatives is applied.

% get axon parameters
p = axonparams;

% Initial values to dependent variables
i = 1:p.lx;
V0 = 0;      y0((1-1)*(p.lx) + i) = V0;  % membrane potential
n0 = 0.3177; y0((2-1)*(p.lx) + i) = n0;  % activation in K channel 
m0 = 0.0529; y0((3-1)*(p.lx) + i) = m0;  % activation in Na channel 
h0 = 0.5961; y0((4-1)*(p.lx) + i) = h0;  % inactivation in Na channel

% specifying option: use the default odeplot function for control 
% purpose, to see progress of the running process. This control  
% graph shows the time profile of membrane potential V(t,x) at x=0 
absTolerance = ones(1, 4*p.lx)*1e-6;
options = odeset('Events', @fevents, 'OutputFcn',@odeplot, ...
    'OutputSel',1, 'RelTol',1e-6, 'AbsTol', absTolerance); 
p.tmax = 5;             % simulation time              [ms] 
%p.tmax = 60;            % simulation time, long simuli [ms]                 
tspan  = [0 p.tmax];    % integration interval         [ms]
% prepare data for graphs
timePoints = [];    
VnmhPoints = [];
Istim = [];
ie = -1;               % any non-empty value to avoid 't=0' as event
while ~isempty(ie) 
    [t, y, te, ye, ie]= ode15s(@derivatives, tspan, y0, options, p);
    timePoints = [timePoints; t];           %#ok<AGROW>
    VnmhPoints = [VnmhPoints; y];           %#ok<AGROW>
    Istim = [Istim; stimulus(t,p)];         %#ok<AGROW>
    tspan = [te(end) p.tmax];
    y0 = ye(end,:);
    % check if an event is detected
    ie = ie(te == te(end));     % eliminate already handled events
    for iEvent = ie'
        switch iEvent
            case 1             % event -> stop stimualtion 
                p.Iinj = 0;
            case 2             % event -> end of observation
                ie = []; 
            otherwise
                % no action needed - placeholder for other events
        end;
    end
end
% extract values for creating graphs     
V(:,i) = VnmhPoints(:,(1-1)*(p.lx) + i);
n(:,i) = VnmhPoints(:,(2-1)*(p.lx) + i);
m(:,i) = VnmhPoints(:,(3-1)*(p.lx) + i);
h(:,i) = VnmhPoints(:,(4-1)*(p.lx) + i);

% Save state of action potential at selected locations x in axon 
xLocations = 1:round(p.lx/20):p.lx-1;   % points along axon  
for ix = xLocations
    ind = find(ix==xLocations);
    figSave.stateV(ind) = figure;
    set(figSave.stateV(ind), 'Position',[100+ix 50 560 420]);
    subplot(2, 1, 1);
    plot(timePoints, V(:,ix), 'LineWidth', 2);
    xlabel('Time [ms]');
    ylabel('V [mV]');
    x = num2str((ix - 1)*p.dx);
    title(['x=', num2str(x), 'cm'])
    subplot(2,1,2);
    plot(timePoints,[n(:,ix) m(:,ix) h(:,ix)], 'LineWidth', 2);
    xlabel('Time [ms]');
    ylabel('Probability of n,m,h');
    legend('n','m','h');
end
% Save state of action potential at all locations x in axon 
figSave.allstateV = figure;
set(figSave.allstateV, 'Position',[100+ix+1 50 560 600]);
for ix = xLocations
    subplot(7, 1, 1:3);
    plot(timePoints, V(:,ix), 'b', 'LineWidth', 2); 
    xlabel('Time [ms]');
    ylabel('V [mV]');
    hold on;
    subplot(7,1, 4:6);
    plot(timePoints,[n(:,ix) m(:,ix) h(:,ix)], 'LineWidth', 2);
    xlabel('Time [ms]');
    ylabel('Probability of n,m,h');
    legend('n','m','h');
    hold on;
end
subplot(7, 1, 7);
plot(timePoints, Istim, 'b', 'LineWidth', 2);
xlabel('Time [ms]');
ylabel('I [\muA]');
ylim([0 max(Istim)+1]);

% Plot membrane potential V(t,x) as surface over time and distance 
figSave.solutionV = figure; set(axes, 'FontSize',15)
set(figSave.solutionV, 'Position',[190+p.lx 50 560 420]);
surf(p.x,timePoints,V, 'FaceColor','interp',...
'EdgeColor','none',...
'FaceLighting','phong');
colorbar
axis tight    
title('Cable Equation: Solution V(x,t)');
xlabel('Distance [cm]');
ylabel('Time [ms]');
zlabel('Voltage [mV]');
% print all figures in TIFF format
printFig(figSave)

end


function p = axonparams
%AXONPARAMS setup axon parameters
%   P = AXONPARAMS creates a structure |P| with the physiological 
%   parameters of the neuron/axon

% Axon parameters
p.a    = 0.0238;       % axon radius                       [cm]
p.L    = 6;            % axon length                       [cm]
p.dx   = 0.05;         % distance integration step         [cm]  
p.x    = 0:p.dx:p.L;   % steps along axon                  [cm] 
p.lx   = length(p.x);  % number of steps within axon 
p.R    = 34.5;         % resistance (axoplasm)             [ohm*cm]
p.C    = 1;            % membrane capacitance              [uF/cm^2]
p.gK   = 36;           % max. potassium (K) conductance    [mS/cm^2]
p.gNa  = 120;          % max. sodium (Na) conductance      [mS/cm^2]
p.gL   = 0.3;          % leak (L) conductance              [mS/cm^2]
p.EK   = -12;          % K equilibrium potential           [mV] 
p.ENa  = 115;          % N aequilibrium potential          [mV]
p.EL   = 10.598;       % L equilibrium potential           [mV]
p.Iinj = 10;           % injected stimulation current      [uA]
p.td   = 0.2;          % duration of injected current      [ms]
% p.td    = 20;        % duration for long stimulus        [ms]
p.T     = 18.5;                 % temperature              [Celsius]
% p.T    = 6.3;                 % temperature              [Celsius]
p.phi  = tempfactor(p.T);       % temperature factor

end


function dydt = derivatives(t, y, p)  % Right side(s) function
%DERIVATIVES Compute the right-hand side of the ODE.
%   DYDT = DERIVATIVES(T, Y, P) calculates |DYDT|, the right-hand
%   side of the ODE model, at points defined by the vector of
%   dependent variables |Y|, time |T|, and with parameters |P|.

i=1:p.lx;
V = y((1-1)*(p.lx) + i);
n = y((2-1)*(p.lx) + i);
m = y((3-1)*(p.lx) + i);
h = y((4-1)*(p.lx) + i);
% boundary condition at the begin of axon (x = 0)
Istim = stimulus(t, p);
Im1 = (Istim/(2*pi*p.a) - p.a/p.R/p.dx*(V(1)-V(2))*1000)/p.dx;
dVdt(1) = (1/p.C)*(Im1 - p.gK*n(1)^4*(V(1)-p.EK) ...
    - p.gNa*m(1)^3*h(1)*(V(1)-p.ENa) - p.gL*(V(1)-p.EL));
dndt(1) = nderiv(n(1), V(1), p.phi);
dmdt(1) = mderiv(m(1), V(1), p.phi);
dhdt(1) = hderiv(h(1), V(1), p.phi);
% computation between the begin and end of axon (0 < x < L)   
i=2:p.lx-1;
Imi = p.a./(p.R*2).*(V(i-1)-2*V(i)+V(i+1))*1000/(p.dx.^2); 
dVdt(i) = (1/p.C)*(Imi - p.gK*n(i).^4.*(V(i)-p.EK) ...
     - p.gNa.*m(i).^3.*h(i).*(V(i)-p.ENa) - p.gL.*(V(i)-p.EL));       
dndt(i) = nderiv(n(i), V(i), p.phi); 
dmdt(i) = mderiv(m(i), V(i), p.phi);
dhdt(i) = hderiv(h(i), V(i), p.phi);    
% boundary condition at the end of axon (x = L) 
Iml = p.a/(p.R*2)*(V(p.lx-1) - 2*V(p.lx))*1000/(p.dx^2); 
dVdt(p.lx) = (1/p.C)*(Iml - p.gK*n(p.lx)^4*(V(p.lx)-p.EK)...
    - p.gNa*m(p.lx)^3*h(p.lx)*(V(p.lx)-p.ENa) ... 
    - p.gL*(V(p.lx)-p.EL));
dndt(p.lx) = nderiv(n(p.lx), V(p.lx), p.phi);
dmdt(p.lx) = mderiv(m(p.lx), V(p.lx), p.phi);
dhdt(p.lx) = hderiv(h(p.lx), V(p.lx), p.phi); 
     
dydt = [dVdt(1)             % (x = 0)   
        dVdt(i)'            % (0 < x < L)   
        dVdt(p.lx)          % (x = L) 
        dndt(1)             % (x = 0)                    
        dndt(i)'            % (0 < x < L)   
        dndt(p.lx)          % (x = L)
        dmdt(1)             % (x = 0)   
        dmdt(i)'            % (0 < x < L)     
        dmdt(p.lx)          % (x = L)
        dhdt(1)             % (x = 0)   
        dhdt(i)'            % (0 < x < L)  
        dhdt(p.lx)];        % (x = L)    
end


function dndt = nderiv(n, V, phi)
%NDERIV Time derivative of |N|.
%   DNDT = NDERIV(N, V, PHI) takes 2 arguments: |V| - membrane  
%   potential, |N| - concentration of proteins activating (opening) 
%   K channel and |PHI| - temperature factor. It returns the 
%   derivative value of |N|.

% Calculate values of the forward and backward rates: alpha, beta 
alpha = (0.01.*(-V + 10))./(exp((-V + 10)/10) - 1)*phi;
beta = 0.125*exp(-V/80)*phi;
dndt = alpha - (alpha + beta).*n;

end


function dmdt = mderiv(m, V, phi)
%DMDT Time derivative of |M|.
%   DMDT = MDERIV(M, V, PHI) takes 2 arguments: |V| - membrane  
%   potential, |M| - concentration of proteins activating (opening) 
%   Na channel and |PHI| - temperature factor. It returns the 
%   derivative value of |M|.  

% Calculate values of the forward and backward rates: alpha, beta
alpha = 0.1*(-V + 25)./(exp((-V + 25)/10) - 1)*phi;
beta = 4*exp(-V/18)*phi;
dmdt = alpha - (alpha + beta).*m;

end


function dhdt = hderiv(h, V, phi)
%HDERIV Time derivative of |H|.
%   DHDT = HDERIV(H, V, PHI) takes 2 arguments: |V| - membrane 
%   potential, |H| - concentration of proteins inactivating (closing)
%   Na channel and |PHI| - temperature factor. It returns the 
%   derivative value of |H|.

% Calculate values of the forward and backward rates: alpha, beta
alpha = 0.07*exp(-V/20)*phi;
beta = 1./(exp((-V + 30)/10) + 1)*phi;
dhdt = alpha - (alpha + beta).*h;

end

function phi = tempfactor(T)
%TEMPFACTOR Compute the temperature factor  
%   PHI = TEMPFACTOR(T) calculates the temperature factor |PHI| to  
%   be considered in HH model if the temperature |T| is different  
%   than 6.3 Celsius.   

phi = 3^((T-6.3)/10);

end


function [value, isterminal, direction] = fevents(t, ~, p)
%FEVENTS Specify events of the ODE model. 
%   [VALUE,ISTERMINAL,DIRECTION] = FEVENTS(T,Y,P) detects the exact  
%   time points of events specified as a function of time |T|,  
%   vector of dependent variables |Y|, and a structure of some ) 
%   (specific) other parameters |P|. 
%
%   Output parameters: |VALUE| - value investigated as event;
%   |ISTERMINAL| = 0|1 where 0 stands for 'don't stop', 1 for 'stop
%   integration'; |DIRECTION| = -1|0|1 where (-1) means 'negative 
%   direction only', 0 - 'both direction', 1 - 'positive direction 
%   only'

% 1 Event: Locate time to stop current injection (stimuli)  
value(1,1)      = t - p.td;   % detect p.td (time to stop current) 
isterminal(1,1) = 1;          % interrupt integration
direction(1,1)  = 1;          % positive direction only
% 2 Event: Locate time to stop observation/integration
value(2,1)      = t - p.tmax; % detected p.tmax (end of observation)
isterminal(2,1) = 1;          % interrupt integration    
direction(2,1)  = 1;          % positive direction only

end


function I = stimulus(t, p)
%STIMULUS Stimulus initialization  
%   I = STIMULUS(T,P) calculates the stimulus at time |T|, and with 
%   parameters |P|; here, |P.Iinj| - is the current amplitude  

I = p.Iinj*(ones(length(t),1));

end


function printFig(fig)
%PRINTFIG Print figures to files
%   PRINTFIG(FIG) saves in TIFF format graphs created during 
%   simulation in external files. |FIG| is a structure containing
%   names of all figures to be saved/printed

for f = fig.stateV
    x = find(f == fig.stateV);
    fileName = ['stateVx_', num2str(x), '.tif'];
    print(f, '-r600', '-dtiff', fileName);
end
print(fig.allstateV, '-r600', '-dtiff', 'allstateV.tif');
print(fig.solutionV, '-r600', '-dtiff', 'solutionVxt');

end

