function ibandronate
%IBANDRONATE select regimens for simulations 
%   IBANDRONATE creates a list-selection dialog box and lets the 
%   user select from a list which regimens type are to be simulated. 
%   No input parameters are needed. This function has to be called
%   first to start the simulation process. 

% Create a list-selection dialog box
listAdmins = {'po25oad', 'po100q1m', 'po150q1m', 'placebo', ...
    'placebo(fast)', 'int','iv1000q1ms', 'test'}; 
selectionNum = listdlg('Name', 'Dose Administation',    ... 
    'PromptString', 'Select dose administation(s):',    ... 
    'SelectionMode', 'multiple', 'ListSize', [180,300], ...
    'ListString', listAdmins);             
selection = listAdmins(selectionNum);  % vectorized selection names   
allreg(selection);

% save graphs as TIFF files
uctxFigure = figure(1);
print(uctxFigure, '-r900', '-dtiff', 'uctx')
ctxAUCFigure = figure(2);
print(ctxAUCFigure, '-r900', '-dtiff', 'ctxAUC')
    
end  


function allreg(selection)
%ALLREG call simualtion for all regimens 
%   ALLREG(SELECTION) initiates simulation for all drug
%   administations specified in |SELECTION|.
%   Examples for calling:
%      allreg({'po150q1m'})
%      allreg({'po100q1m', 'po25oad', 'placebo'})
%      allreg({'po150q1m', 'po100q1m', 'po25oad', 'placebo'})
    
hpw = waitbar(0,'Please wait...');
uctxFigure = figure(1); set(uctxFigure, 'Visible', 'off')
uctxAxes = axes; set(uctxAxes, 'FontSize',15)
ctxAUCFigure = figure(2); set(ctxAUCFigure, 'Visible', 'off')
ctxAUCaxes = axes; set(ctxAUCaxes, 'FontSize',15)

numSimulations = length(selection);           
uctxLegend = cell(numSimulations, 1);
ctxAUCLegend = cell(numSimulations, 1);
for i=1:numSimulations 
    [rAUC, info] = solveiban1(selection{i}, hpw);
    display(info);
    uctxLegend{i} = num2str(selection{i});
    ctxAUCLegend{i} = num2str(rAUC);
end
close(hpw)
set(uctxFigure,'units','normalized','Position', ...
[0.1, 0.5, 0.3, 0.4] ,'Visible','on'); 
title('uCTX vs. Time');
legend(uctxLegend, 'Location','East');
set(ctxAUCFigure, 'units','normalized','Position', ... 
    [0.6, 0.5, 0.3, 0.4] ,'Visible','on');
title('CTX vs. Time / AUC (legend)');
legend(ctxAUCLegend, 'Location','East');  
 
end 


function [AUC, outText] = solveiban1(regimenType,hpw)
%SOLVEIBAN1 Solve ibandronate model. 
%   [AUC, OUTTEXT] = SOLVEIBAN1(REGIMENTYPE,HPW) solves the
%   ibandronate model for 1 regimen, specified as |REGIMENTYPE|. 
%   |HPW| is the handle to the wait bar that has to display the     
%   progress of calculation. The function returns the solution in 
%   graphical form, area under the curve |AUC|, and a message,
%   |OUTTEXT|, if successfully passed.
 
% model parameters / initialization
p.rin    = 255;        % formation rate
p.kout   = 1.06;       % degradation rate 
p.slope  = 2.32E-4;    % slope contribution of placebo effect
p.VIT    = 0.314;      % vitamin/ca++ effect
p.KVIT   = 0.0118;     % rate constant
p.VITDCA = 1;          % Vit.D Calc yes(1)/no(0)
p.HILL   = 0.913;      % shape factor   
p.Pdr50  = 17.2;       % potency for inhibition  
p.k1     = 0.0138;     % elimination rate constant +VITD/Ca++              
p.k2     = 0.112;      % elimination rate constant -VITD/Ca++              
p.Y20    = p.rin/p.kout;
p.k      = p.k1*p.VITDCA+p.k2*(1-p.VITDCA);

% dose time and dose quntity preparation
[p.doseTimes, p.doseAmounts, sCol] = doseSchedule(regimenType);
p.numDoses = length(p.doseTimes);   % number of doses
i=1:p.numDoses-1;
doseIndexes = num2cell(i);
% preparation for time domain: [0; t(1)]
tspan = [0 1200];
options = odeset('Events',@events);
% Initial values (stage 1)
amount = p.doseAmounts(1);            % 1-st dose = initial value;  
y0 = [amount; p.Y20; 0];       
[t,y,~,~,iE] = ode45(@derivatives, tspan, y0, options, p);
% Prepare table for graph  
timePoints = t;    
yPoints = y; 

% Handle Events 
while ~isempty(iE)  
    waitbar(iE(end)/p.numDoses, hpw, ['Please wait...', ...
        regimenType]);
    switch iE(end)
        case doseIndexes       % event is a dose time 
            iE(end);
            % preparation for time domain: [t(n); t(n+1)]
            tspan = [t(end), t(end) + 1:1200]; 
            amount = p.doseAmounts(iE(end));        
            % Initial values (stage after event) 
            y0 = [y(end,1) + amount, y(end,2:end)];
            [t,y,~,~,iE] = ode45(@derivatives, tspan, y0, ... 
                options, p);
            timePoints = [timePoints; t];      %#ok 
            yPoints = [yPoints; y];  %#ok Prepares table for graphic 
        case p.numDoses     % end of time integration event      
            break;     
        otherwise
            messageID = 'Book:ibandronate:UnknownEvent';
            messageStr = 'Unknown event ''%s''.';
            error(messageID, messageStr, num2str(iE(end)));
    end
end 

% Compute Area Under the Curve (AUC) 
AUC = yPoints(end,3);
% Prepare graphs
numPoints = length(timePoints);
ctx = zeros(numPoints,1); 
pctctx = zeros(numPoints,1);

i=1:numPoints;
PBO = 1+p.slope*timePoints(i,1);  %  baseline fraction due to disease
VITD = (1-p.VIT.*(1-exp(-p.KVIT.*timePoints(i,1)))).^p.VITDCA;
ctx(i,1) = yPoints(i,2).*PBO.*VITD;
pctctx(i,1) = (ctx(i,1)-p.Y20)./p.Y20.*100;

figure(1); 
% set(uctxFigure, 'Visible', 'off')
plot(timePoints, pctctx, sCol,'LineWidth', 2); 
xlabel('Time [d]'); 
ylabel('uCTX_{ PCTBS}');
grid on; hold on;

figure(2); 
% set(ctxAUCFigure, 'Visible', 'off');    
plot(timePoints, ctx, sCol, 'LineWidth', 2); 
xlabel('Time [d]'); 
ylabel('CTX'); 
grid on; hold on; 
outText = ['regimen: ', num2str(regimenType), ' passed'];

end


function [doseTimes, doseAmounts, sCol] = doseSchedule(regimenType)
%DOSESCHEDULE Creates a vector of dosing times and amounts of drug.
%   [DOSETIMES,DOSEAMOUNT,SCOL] = DOSESCHEDULE(REGIMENTYPE) returns 
%   a vector |DOSETIMES| of dosing times and a scalar |DOSEAMOUNT| 
%   of the dosing amount. |REGIMENTYPE| must be either 'po25oad',   
%   'po100q1m', 'po150q1m', 'placebo', 'placebo(fast)', 'int',
%   'iv1000q1ms', or 'test'. |SCOL| is a colour option the type of
%   dosing regimen.   

switch regimenType   
    case 'po25oad'              
        sCol = 'b';         % blue          
        dose = 2500;        % microgram
        F1  = 0.00757;      % bioavailability
        doseTimes = 0:1:1080;
        doseAmounts = ones(1,length(doseTimes))*dose*F1;        
    case 'po100q1m'
        sCol = 'g';         % green 
        dose = 100000;      % microgram 
        F1  = 0.00664;      % bioavailability
        doseTimes= 0:30:1080;
        doseAmounts = ones(1,length(doseTimes))*dose*F1; 
    case 'po150q1m'
        sCol = 'r';         % red
        dose = 150000;      % microgram
        F1  = 0.00664;      % bioavailability
        doseTimes = 0:30:1080;
        doseAmounts = ones(1,length(doseTimes))*dose*F1;       
    case 'placebo'
        sCol = 'k';         % black
        % no dose neither F1 as it is placebo (dose = 0) 
        doseTimes = 0:1080;
        doseAmounts = zeros(1,length(doseTimes));   
    case 'placebo(fast)' 
        sCol = 'c';         % cyan
        % no dose neither F1 as it is placebo (dose = 0) 
        doseTimes = [0 1080];
        doseAmounts = zeros(1,length(doseTimes));
    case 'int' 
        sCol = 'y';         % yellow
        dose = 20000;       % microgram
        F1  = 0.00664;      % bioavailability
        % dose given on day:   0,   2, ... ,  22  
        % and then            90,  92, ... , 112 
        % and then           180, 182, ... , etc. 
        doseTimes = reshape(repmat((0:2:22)',1,length(0:90:990)) ...
            + repmat(0:90:990, length(0:2:22), 1), 1, ... 
            length(0:90:990)*length(0:2:22));
        doseAmounts = ones(1,length(doseTimes))*dose*F1;      
    case 'iv1000q1ms'
        sCol = 'c';         
        dose = 1000;        % ug !!!!!
        F1  = 1;
        doseTimes = 0:30:1080;
        doseAmounts = ones(1,length(doseTimes))*dose*F1;
    case 'test'             % another test example 
        sCol = 'm';         % magenta
        doseTimes   = [  0 125 500 600 620 1000 1080];
        doseAmounts = [664 664 664 664 664  664  664];
    otherwise
        messageID = 'Book:ibandronate:UnknownRegimen';
        messageStr = ['Unknown regimen ''%s''. A regimen must ', ...
            'be: ''po25oad'', ''po100q1m'', ''po150q1m'',', ...
            '''placebo'', ''placebo(fast)'', ''int'', ', ...
            '''iv1000q1ms'' or ''test''.'];
        error(messageID, messageStr, regimenType);
end
doseTimes = doseTimes';     
doseAmounts = doseAmounts'; 

end 


function dydt = derivatives(t, y, p)
%DERIVATIVES Compute the right-hand side of the ODE.
%   DYDT = DERIVATIVES(T, Y, P) calculates |DYDT|, the right-hand
%   side of the ODE model, at points defined by the vector of
%   dependent variables |Y|, time |T|, and with parameters |P|.

dydt(1) = -p.k*y(1);                                            
dydt(2) = p.rin*(1 - (y(1)*p.k)^p.HILL/(p.Pdr50^p.HILL    ...
          +(y(1)*p.k)^p.HILL)) - p.kout*y(2);           
dydt(3) = y(2)*(1 + p.slope*t)*(1 - p.VIT*(1-exp(-p.KVIT*t)))  ... 
          ^p.VITDCA;                                           
dydt = dydt';               % must be a column vector

end 


function [value,isterminal,direction] = events(t,~,p)
%FEVENTS Specify events of the ODE model. 
%   [VALUE,ISTERMINAL,DIRECTION] = FEVENTS(T,Y,P) detects the exact  
%   time points of events specified as a function of time |T|,  
%   vector of dependent variables |Y|, and a structure of some ) 
%   (specific) other parameters |P|. 
%
%   Output parameters: |VALUE| - value investigated as event;
%   |ISTERMINAL| = 0|1 where 0 stands for 'don't stop', 1 for 'stop
%   integration'; |DIRECTION| = -1|0|1 where (-1) means 'negative 
%   direction only', 0 - 'both direction', 1 - 'positive direction 
%   only'

% Locate the time when "t - <event-time>" passes 0
% dose time - events
value(:,1) = t - p.doseTimes(2:p.numDoses)';     % time event values 
isterminal(:,1) = ones(1,p.numDoses - 1);        % [1 1 ... ]
direction(:,1) = zeros(1,p.numDoses - 1);        % [0 0 ... ]
% time integration - event
value(p.numDoses,1) = t - 1081;   % end of time integration
isterminal(p.numDoses,1) = 1;     % Stop the integration definitely
direction(p.numDoses,1) = 0;      % both direction
    
end  


