function infus1com()
%INFUS1COM() One-compartment model with infusion
%   INFUS1COM() computes plasma concentration versus time course 
%   and creates plots for a one-compartment model. Drug doses were 
%   applied as infusions. The function has no parameters.

% PK parameter(s) specification
p.Dose  = 200;              % dose [mg]
p.tStop = 2;                % infusion time stop [h]
p.R     = p.Dose/p.tStop;   % infusion rate [mg/h]
p.V     = 12.6;             % volume of distribution [L]
p.CL    = 2.16;             % clearance [L/h]

options = odeset('Events', @fevents, 'RelTol',1e-3, 'AbsTol', 1e-3);
ie    = 99;         % any non-empty value to avoid 't=0' as event 
tspan = [0,100];    % integration time interval
c0    = 0;          % initial concentration value
% prepare data for graphs
timePoints = []; concPoints = []; ratePoints = [];
% Call differential equation solver (Runge-Kutta Method) 
% and check events 
while ~isempty(ie)      
    [t,c,te,ce,ie] = ode45(@derivatives, tspan, c0, options, p);
    % collect data to generate graph 
    timePoints = [timePoints; t];   %#ok<AGROW>
    concPoints = [concPoints; c];   %#ok<AGROW>
    ratePoints = [ratePoints; p.R*ones(length(t),1)];   %#ok<AGROW>
    tspan = [te(end) 100]; 
    c0 = ce(end,1);
    % check if an event is detected
    switch ie(end)
        case 1      % event: stop infusion
            p.R = 0;
        case 2      % event: end of observation
            ie = []; 
        otherwise
            % no action needed - placeholder for other events
    end;   
end

% Generating graphs
subplot(3, 1, 1)
plot(timePoints, ratePoints,  '-k', 'LineWidth', 2); hold on 
title({' ', '\fontsize{13}One-Compartment Model With Infusion', ''})
ylim([0 110])
ylabel('\fontsize{13}Rate [mg/h]')
subplot(3, 1, 2:3); 
plot(timePoints, concPoints, '-k', 'LineWidth', 2  )
 hold on; grid minor; grid on ; 
xlabel('\fontsize{13}Time [hour]')
ylabel('\fontsize{13}Concentration [mg/L]')
% save graphs as TIFF file 
print('-r900', '-dtiff', 'infus1com')

end


function dcdt = derivatives(t, c, p) 
%DERIVATIVES Compute the right-hand side of the ODE.
%   DCDT = DERIVATIVES(T, C, P) calculates |DCDT|, the right-hand
%   side of the ODE model, at points defined by the vector of
%   dependent variables |C|, and time |T|, and with parameters |P|.

dcdt = r(t,p)/p.V - p.CL/p.V*c;
   
end


function rt = r(~, p)
%R Specifies Infusion Rate
%   RT = R(T,P) returns infusion rate |RT| at time |T|, and with
%   parameters |P|

% infusion (depends on event)
rt = p.R; 
  
end 


function [value, isterminal, direction] = fevents(t, ~, p)
%FEVENTS Specify events for the ODE model. 
%   [VALUE,ISTERMINAL,DIRECTION] = FEVENTS(T,Y,P) detects the exact  
%   time points of events specified as a function of time |T|,  
%   vector of dependent variables |Y|, and a structure of some ) 
%   (specific) other parameters |P|. 
%
%   Output parameters: |VALUE| - value investigated as event;
%   |ISTERMINAL| = 0|1 where 0 stands for 'don't stop', 1 for 'stop
%   integration'; |DIRECTION| = -1|0|1 where (-1) means 'negative 
%   direction only', 0 - 'both direction', 1 - 'positive direction 
%   only'

% 1. Event: Locate the time to stop infusion   
value(1,1)      = t - p.tStop; % Detect t = 2 (infusion stop time) 
isterminal(1,1) = 1;           % Interrupt integration, stop infusion
direction(1,1)  = 1;           % Positive direction only

% 2. Event: Locate the time to stop observation/integration
value(2,1)      = t - 15 ;    % Detect t = 15 (end of observation)
isterminal(2,1) = 1;          % Stop integration    
direction(2,1)  = 1;          % Positive direction only

end
