function infus1comstep()
%INFUS1COMSTEP() One-compartment model with infusion.
%   INFUS1COMSTEP() computes plasma concentration-time course,
%   creates graphs for different error tolerances, specified by
%   parameters 'RelTol (relative tolerance) and 'AbsTol' (absolute
%   tolerance), and compares the numerical solutions with the exact
%   (analytical) solution. In the treatment, 200 [mg] doses is given
%   as 2-h infusions to each subject. Time courses of plasma
%   concentration and infusion are shown in a generated graph.
 
% to see 'zoom' set: zoom = 1; otherwise zoom = 0
zoom = 0;
if zoom 
    xzoom = [1.7 3];                                         % ZOOM 
    trt = 0:0.001:3;                                         % ZOOM
    tx = ['One-Compartment Model with Infusion (zoom)', 10]; % ZOOM
    ftif = 'LOWER_PANEL_zoom.tif';                           % ZOOM
else
    xzoom = [0 15];                                     % NO ZOOM 
    trt = 0:0.001:15;                                   % NO ZOOM 
    tx = ['One-Compartment Model with Infusion', 10];   % NO ZOOM 
    ftif = 'UPPER_PANEL_norm.tif';                      % NO ZOOM
end
% PK parameter(s) specification
p.V  = 12.6;    % Volume of distribution
p.CL = 2.16;    % Clearance
 
% Generating graphs
 
subplot(3, 1, 1)
plot(trt, r(trt), '-b', 'LineWidth', 2); hold on 
ylim([0 110])
xlim(xzoom)
ylabel({'Infusion' 'Rate [mg/h]'}, 'FontSize', 12)
title(tx, 'FontSize', 15)
 
subplot(3,1,2:3);
% Calling the differential equation solver (Runge-Kutta Method) 
% with different tolerance parameters
options = odeset('RelTol',1e-3, 'AbsTol',1e-6);
    tspan = 0:0.2:15;                                       
[timePoints,concPoints] = ode45(@derivatives, tspan, 0, options, p);
plot(timePoints, concPoints, '+b', 'LineWidth', 2); hold on 
 
options = odeset('RelTol',1e-6, 'AbsTol',1e-6);
[timePoints,concPoints] = ode45(@derivatives, tspan, 0, options, p);
plot(timePoints, concPoints, 'og', 'LineWidth', 2); hold on 
 
options = odeset('RelTol',1e-3, 'AbsTol',1e-4);
[timePoints,concPoints] = ode45(@derivatives, tspan, 0, options, p);     
plot(timePoints, concPoints, 'sr', 'LineWidth', 2); hold on  
 
options = odeset('RelTol',1e-3, 'AbsTol',1e-7);
[timePoints,concPoints] = ode45(@derivatives, tspan, 0, options, p);    
plot(timePoints, concPoints, 'dm', 'LineWidth', 2); hold on  
 
concPoints = concentration(tspan, p);
plot(tspan, concPoints, 'k', 'LineWidth', 2); hold on
 
grid('minor')
xlim(xzoom)
xlabel('Time [h]', 'FontSize', 12)
ylabel('Concentration [mg/L]', 'FontSize', 12)
legend('[A]','[B]','[C]','[D]','[E]')
if ~zoom 
    plotell(2.5, 11.5, 1.6667, 3.2258);
    text(4.5, 11.5, 'zoom circle', 'FontSize', 12 )
end;
 
print('-r900', '-dtiff', ftif)       
 
end
 

function dcdt = derivatives(t, c, p) 
%DERIVATIVES Compute the right-hand side of the ODE.
%   DYDT = DERIVATIVES(T, C, P) calculates |DCDT|, the right-hand
%   side of the ODE model, at points defined by the vector of
%   dependent variables |C|, time |T|, and with parameters |P|.
 
dcdt = r(t)/p.V - p.CL/p.V*c;  
 
end
 

function rt = r(t)
%R Specifies Infusion Rate  
% RT = R(T) returns infusion rate |RT| at time |T|  
 
R = 200/2;
rt = R * (t < 2); 
 
end
 

function c = concentration(t, p)
% CONCENTRATION computes plasma concentration 
%   C = CONCENTRATION(T, P) computes the exact value of concentration
%   based on the direct formula derived from Laplace transform method
%   Input: |T| - time and |P| - structure with model parameters 
 
% Use analytical solution (as string)
R = 200/2;
cstr = 'R/CL - (R*exp(-(CL*t)/V))/CL + R*heaviside(t - 2)*(exp(-(CL*(t - 2))/V)/CL - 1/CL)';
c = subs(cstr,{'R' , 'V' , 'CL', 't'}, {R , p.V , p.CL, t});
end
 

function plotell(x0, y0, a, b)
%PLOTELL Plot an ellipse.
%   PLOTELL(X0, Y0, A, B) plots a ellipse with the center in |X0|,|Y0|
%   and semiaxes |A|,|B|.
 
t=0:.01:2*pi; 
plot(sin(t)*a + x0, cos(t)*b + y0, 'r', 'LineWidth', 2.5);
 
end
