function infus1inter()
%INFUS1INTER() One-Compartment Model with Infusion
%   INFUS1INTER() computes plasma concentration-time course and     
%   creates graphics output for one-compartment model. A 200 [mg]    
%   dose was given as 2-h infusion. Time courses of plasma 
%   concentration and infusion rate are plotted. The function has 
%   two parameters, volume (V) and clearance (CL)

% PK parameter(s) specification
p.V  = 12.6;    % Volume of distribution
p.CL = 2.16;    % Clearance

% Calling the differential equation solver (Runge-Kutta Method)                           
[t,c] = ode45(@derivatives, 0:0.01:15, 0, [], p); 
 
% Generating graphs
subplot(3, 1, 1); 
plot(t, r(t), 'Color', 'k', 'LineWidth', 2)
title(['One-Compartment Model with Infusion', 10], 'FontSize', 15)
ylim([0 110])
ylabel({'Infusion' 'Rate [mg/h]'},'FontSize', 12)
subplot(3, 1, 2:3); 
plot(t, c, 'Color', 'k', 'LineWidth', 2)   
xlabel('Time [h]', 'FontSize', 12)
ylabel('Concentration [mg/L]', 'FontSize', 12)
% save graphs as TIFF file 
print('-r900', '-dtiff', 'infus1inter')

end


function dcdt = derivatives(t, c, p) 
%DERIVATIVES Compute the right-hand side of the ODE.
%   DCDT = DERIVATIVES(T, C, P) calculates |DCDT|, the right-hand
%   side of the ODE model, at points defined by the vector of
%   dependent variables |C|, time |T|, and with parameters |P|.

dcdt = r(t)/p.V - p.CL/p.V*c; 

end


function rt = r(t)
%R Specifies Infusion Rate  
%   RT = R(T) returns infusion rate |RT| at time |T|  

rt = 200/2 * (t < 2);

end
