function infus2com()
%INFUS2COM() Two-compartment model with infusion.
%   INFUS2COM() computes plasma concentration time course and creates    
%   graph for two-compartment model. In the treatment, doses were  
%   given as infusions to each subject. A graph with time courses of 
%   plasma concentration and infusion is generated. The function has 
%   no arguments.

p.Rate = 50;         % initial infusion rate = 100/2 [mg/h] 
p.CL   = 1.38E+01;   % central clearance
p.V1   = 1.48E+01;   % volume of distribution in central compartment 
p.Q    = 2.17E+00;   % inter-compartmental clearance
p.V2   = 4.23E+00;   % volume of distribution peripheral compartment
p.k    = p.CL/p.V1;  % rate constant of elimination              
p.k12  = p.Q/p.V1;   % rate constant from central to peripheral             
p.k21  = p.Q/p.V2;   % rate constant from peripheral to central           

options = odeset('Events',@fevents, 'RelTol',1e-6, 'AbsTol', ...
    [1e-6 1e-6]); 
% Initial values
tspan = [0 50];     % max. time domain
c0 = [0 0];         % Initial value for the dependent variable
ie = 99;            % avid empty events vector
while ~isempty(ie)    
    % Calling Runge-Kutta Method
    [t,c,te,ye,ie] = ode45(@derivatives, tspan, c0, options, p);
    % Generate graph 
    plot(t, c(:,1), '-', 'Color', 'r', 'LineWidth', 2)
    hold on;
    plot(t, c(:,2), '-.', 'Color', 'b', 'LineWidth', 2)
    tspan = [te(end) 50]; 
    c0 = [ye(end,1) ye(end,2)];
    % check if an event is detected
    switch ie(end) 
        case 1          % event: stop infusion
            p.Rate = 0; 
        case 2          % event: end of observation
            ie = [];
        otherwise 
            % no action needed
    end;   
end
% Describe graph
title('Two-Compartment Model with Infusion')
xlabel('Time [h]');
ylabel('Concentration [mg/L]');
text('units','inch', 'position',[1.9 2.7], ...
'fontsize',11, 'interpreter','latex', 'string',...
['$${dc_1 \over dt} = {r\over V_1} - (k + k_{12})\cdot c_1 + ' ...
'k_{21}\cdot {V_2 \over V_1} \cdot c_2 $$'])
text('units','inch', 'position',[1.9 2.2], ...
'fontsize',11, 'interpreter','latex', 'string', ...
['$${dc_2 \over dt} = k_{12} \cdot {V_1 \over V_2}' ...
'\cdot c_1 - k_{21} \cdot c_2 \   $$'])
legend('central compartment','peripheral compartment')
% save graphs as TIFF file 
print('-r900', '-dtiff', 'infus2com') 
   
end 


function dcdt = derivatives(~, c, p)
%DERIVATIVES Compute the right-hand side of the ODE.
%   DCDT = DERIVATIVES(T, C, P) calculates |DCDT|, the right-hand
%   side of the ODE model, at points defined by the vector of
%   dependent variables |C|, and time |T|, and with parameters |P|.

dcdt = [p.Rate/p.V1 - (p.k+p.k12)*c(1) + p.k21*p.V2/p.V1*c(2) 
        p.k12*p.V1/p.V2*c(1) - p.k21*c(2)                    ];          
end


function [value, isterminal, direction] = fevents(t, ~, ~)
%FEVENTS Specify events for the ODE model. 
%   [VALUE,ISTERMINAL,DIRECTION] = FEVENTS(T,Y,P) detects the exact  
%   time points of events specified as a function of time |T|,  
%   vector of dependent variables |Y|, and a structure of some ) 
%   (specific) other parameters |P|. 
%
%   Output parameters: |VALUE| - value investigated as event;
%   |ISTERMINAL| = 0|1 where 0 stands for 'don't stop', 1 for 'stop
%   integration'; |DIRECTION| = -1|0|1 where (-1) means 'negative 
%   direction only', 0 - 'both direction', 1 - 'positive direction 
%   only'


% 1. Event: Locate the time to stop infusion   
value(1,1) = t - 2;       % Detect t = 2 i.e. time to stop infusion
isterminal(1,1) = 1;      % Stop integration
direction(1,1)  = 1;      % Positive direction only

% 2. Event: Locate the time to stop observation / integration
value(2,1) = t - 15 ;     % Detect t = 15 i.e. end of observation
isterminal(2,1) = 1;      % Stop the integration
direction(2,1) = 1;       % Positive direction only

end


end
   