
function infus2comlapl
%INFUS2COM Two-Compartment Model With Infusion
%   INFUS2COM computes plasma concentration time course and creates    
%   graphs for two-compartment model. In the treatment doses was given   
%   as infusions to each subjects. Time courses of plasma 
%   concentration and infusion are shown in a generated graph
%   is generated.

p.R  = 50;          % initial infusion rate = 100/2 [mg/h] 
p.CL = 1.38E+01;    % central clearance
p.V1 = 1.48E+01;    % volume of distribution in central compartment 
p.Q  = 2.17E+00;    % inter-compartmental clearance
p.V2 = 4.23E+00;    % volume of distribution peripheral compartment

p.k   = p.CL/p.V1;  % rate constant of elimination              
p.k12 = p.Q/p.V1;   % rate constant from central to peripheral             
p.k21 = p.Q/p.V2;   % rate constant from peripheral to central           

options = odeset('Events',@fevents,'RelTol',1e-3,'AbsTol', ...
    [1e-6 1e-6]); 

% Initial values
tspan = [0 50];     % max. interval for time domain
c0 = [0 0];         % initial value for dependent variable
ie = 99;

timePoints = [];
concPoints = [];
while ~isempty(ie)    
    % Calling Runge-Kutta Method
    [t,c,te,ye,ie] = ode23(@derivatives, tspan, c0, options, p);
    % Generate graph 
    timePoints = [timePoints; t];       %#ok<AGROW>
    concPoints = [concPoints; c];       %#ok<AGROW>
    tspan = [te(end) 50]; 
    c0 = [ye(end,1) ye(end,2)];
    % check if an event is detected
    %for ex = ie(end,:)
        switch ie(end)
            case 1          % event: stop infusion
                p.R = 0; 
                display('stop infusion')
            case 2          % event: end of observation
                ie = [];
                display('end of observation')
            otherwise 
                % no action needed
        end;
    %end
end
set(axes,'FontSize',15)
plot(timePoints, concPoints(:,1), 'o', 'Color', 'k', 'LineWidth', 2);
hold on;
plot(timePoints, concPoints(:,2), '+', 'Color', 'k', 'LineWidth', 2);
hold on;

p.R  = 50; 
global x1 x2

c = lapl2com();
x1 = inline(c(1)) %#ok<NOPRT>
x2 = inline(c(2)) %#ok<NOPRT>

t = 0:0.01:15;
ct = concentration(t,p);     % Calling exact solution
timePoints = t;
concPoints = ct;
plot(timePoints, concPoints(:,1), '-', 'Color', 'k', 'LineWidth', 2)
plot(timePoints, concPoints(:,2), '--', 'Color', 'k', 'LineWidth', 1)

title('Two-Compartment Model with Infusion')
xlabel('Time [h]');
ylabel('Concentration [mg/L]');
legend('ODE central compartment', 'ODE perifheral compartment', ...
    'Laplace solution', 'Laplace solution')
print('-r900', '-dtiff', 'infus2comlapl')
   
end


function dcdt = derivatives(~, c, p)
%DERIVATIVES Compute the right-hand side of the ODE.
%   DCDT = DERIVATIVES(T, C, P) calculates |DCDT|, the right-hand
%   side of the ODE model, at points defined by the vector of
%   dependent variables |C|, time |T|, and with parameters |P|.

dcdt = [p.R/p.V1-(p.k+p.k12)*c(1)+p.k21*p.V2/p.V1*c(2); 
        p.k12*p.V1/p.V2*c(1)-p.k21*c(2)                ];          
end


function [value, isterminal, direction] = fevents(t, ~, ~)
%FEVENTS Specify events of the ODE model. 
%   [VALUE,ISTERMINAL,DIRECTION] = FEVENTS(T,Y,P) detects the exact  
%   time points of events specified as a function of time |T|,  
%   vector of dependent variables |Y|, and a structure of some ) 
%   (specific) other parameters |P|. 
%
%   Output parameters: |VALUE| - value investigated as event;
%   |ISTERMINAL| = 0|1 where 0 stands for 'don't stop', 1 for 'stop
%   integration'; |DIRECTION| = -1|0|1 where (-1) means 'negative 
%   direction only', 0 - 'both direction', 1 - 'positive direction 
%   only'

% 1. Event: Locate the time to stop infusion   

value(1,1) = t - 2;   % Detect t = 2 i.e. time to stop infusion
isterminal(1,1) = 1;      % Stop the integration
direction(1,1)  = 1;      % Positive direction only

% 2. Event: Locate the time to stop observation / integration
value(2,1) = t - 15 ;     % Detect t = 15 i.e. end of observation
isterminal(2,1) = 1;      % Stop the integration
direction(2,1) = 1;       % Positive direction only

end


function c = concentration(t, p)
%CONCENTRATION Compute concentration profile.
%   C = CONCENTRATION(T, P) computes the concentration values |C| at
%   point define at time point |T|, and for additional model
%   parameters |P|.


syms c1 c2
global x1 x2

c(:,1) =  x1(p.R, p.V1, p.k, p.k12, p.k21, t); 
c(:,2) =  x2(p.R, p.V2, p.k, p.k12, p.k21,t);

end

   