function infus2comstev  
%INFUS2COMSTEV Two-compartment model with state events
%   INFUS2COMSTEV implements state events in two-compartment model.  
%   Plasma concentration should be kept within a given range by 
%   infusion rate: if drug concentration achieves the maximum then 
%   decrease infusion; if minimum - infusion has to be increased.  

% PK parameters
CL = 1.38E+01;      % clearance 
V1 = 1.48E+01;      % volume of distribution in 1st compartment   
Q  = 2.17E+00;      % intercompartmental clearance  
V2 = 4.23E+00;      % volume of distribution in 2nd compartment 
Rate  = 50;         % infusion rate
% p structure with PK parameters available in several functions 
p.Rate = Rate;
p.V1   = V1;
p.V2   = V2;
p.k    = CL/V1;            
p.k12  = Q/V1;            
p.k21  = Q/V2;           
    
options = odeset('Events',@fevents);
tspan = [0 50];
c0 = [0 0];   % Initial value of concentrations
ie = 99;
while ~isempty(ie)
    [t, c, te, ye, ie] = ode45(@derivatives, tspan, c0, options, p); 
    plot(t, c(:,1), 'Color','r','LineWidth',2)     
    hold on;
    plot(t, c(:,2), 'Color','b','LineWidth',2)
    tspan = [te(end) 50]; 
    c0 = [ye(end,1) ye(end,2)];
    switch ie(end)
        case 1
            % stop infusion after 2 hours 
            p.Rate = 0;       
        case 2
            % stop process definitely
            ie = [];      
        case 3
            % reduce infusion if conc. has grown to allowed max
            p.Rate = Rate/10;     
        case 4
            % allow  full infusion rate if conc. has fallen to min
            p.Rate = Rate;       
        otherwise
            messageID = 'Book:infus2comstev:UnknownEvent';
            messageStr = 'Unknown event ''%s''.';
            error(messageID, messageStr, num2str(ie(end)));
    end
end
hold off;
xlabel('Time [h]');
ylabel('Concentration [mg/L]');
legend('central compartment','peripheral compartment')
title('Two-Compartment Model / Infusion / Multiple Doses') 
% save graphs as TIFF file 
print('-r900', '-dtiff', 'infus2comstev') 

end

function dcdt = derivatives(~, c, p)
%DERIVATIVES Compute the right-hand side of the ODE.
%   DCDT = DERIVATIVES(T, C, P) calculates |DCDT|, the right-hand
%   side of the ODE model, at points defined by the vector of
%   dependent variables |C|, time |T|, and with parameters |P|.

dcdt = [ p.Rate/p.V1 - (p.k + p.k12)*c(1) + p.k21*p.V2/p.V1*c(2)    
         p.k12*p.V1/p.V2*c(1) - p.k21*c(2)                      ];      
end


function [value,isterminal,direction] = fevents(t,y,~)
%FEVENTS Specify events for the ODE model. 
%   [VALUE,ISTERMINAL,DIRECTION] = FEVENTS(T,Y,P) detects the exact  
%   time points of events specified as a function of time |T|,  
%   vector of dependent variables |Y|, and a structure of some ) 
%   (specific) other parameters |P|. 
%
%   Output parameters: |VALUE| - value investigated as event;
%   |ISTERMINAL| = 0|1 where 0 stands for 'don't stop', 1 for 'stop
%   integration'; |DIRECTION| = -1|0|1 where (-1) means 'negative 
%   direction only', 0 - 'both direction', 1 - 'positive direction 
%   only'
  
value(1,1) = t - 2;       % max time for first infusion
isterminal(1,1) = 1;      % Stop the integration
direction(1,1) = 1;       % Positive direction only

value(2,1) = t - 30;      % End of observation
isterminal(2,1) = 1;      % Stop the integration
direction(2,1) = 1;       % Positive direction only

value(3,1) = y(1) - 2;    % Detect concentration = max while growing 
isterminal(3,1) = 1;      % Stop the integration
direction(3,1) = 1;       % Positive direction only

value(4,1) = y(1) - 0.5;  % Detect concentration = min while falling   
isterminal(4,1) = 1;      % Stop the integration
direction(4,1) = -1;      % Negative direction only

end
