function c = lapl2com()
%LAPL2COM Laplace solution for two-compartment model with infusion.
% C = LAPL2COM()solves the equation:
% dcdt = [p.R/p.V1-(p.k+p.k12)*c(1)+p.k21*p.V2/p.V1*c(2); 
%         p.k12*p.V1/p.V2*c(1)-p.k21*c(2)                ];
% using first Laplace transform and then inverse Laplace transform. 
% The result is a vector function |C| = c(t) in analytic form.

syms  R V1 V2 s t k k12 k21 LC1 c1 LC2 c2 Rt Lr r   

dc1dt = sym('diff(c1(t),t)'); 
dc2dt = sym('diff(c2(t),t)'); 
c1 = sym('c1(t)');
c2 = sym('c2(t)');
r = sym('r(t)');
%Rt = R*(heaviside(t) - heaviside(t-2)); 
equation1 = dc1dt - r/V1 + (k+k12)*c1 - k21*V2/V1*c2;
equation2 = dc2dt - k12*V1/V2*c1 + k21*c2;
transformL1 = laplace(equation1, t, s);
transformL2 = laplace(equation2, t, s);
% specify Laplace transform of c(t) as Lc
transformL1 = subs(transformL1, {'c1(0)', 'laplace(c1(t), t, s)', ...
    'laplace(c2(t), t, s)','laplace(r(t), t, s)'}, ...
    {0 , LC1, LC2, Lr});      %#ok<NODEF>
transformL2 = subs(transformL2, {'c2(0)', 'laplace(c1(t), t, s)', ... 
    'laplace(c2(t), t, s)'},{0 , LC1, LC2});     
transformL1 = collect(transformL1, LC1);
transformL2 = collect(transformL2, LC2);
[LC1, LC2] = solve(transformL1, transformL2, LC1, LC2);
Ks = [LC1/Lr; LC2/Lr];
% Show the transform function in 'like algebraic' notation
pretty(Ks)
% specify input (dose administration)
r = R*(heaviside(t) - heaviside(t-2));  % infusion
%r = R*dirac(t);                        % bolus
Us = laplace(r, t, s);
Ys = Ks*Us;
c = ilaplace(Ys, s, t);
c = simplify(c);
pretty(c)
display(['c1(t) = ' char(c(1))]);
display(['c2(t) = ' char(c(2))]);

end
