function laplgraph
%LAPLGRAPH Generate the response to a combined dosing regimen.
%   LAPLGRAPH produces the concentration time profile as response
%   to a dosing regimen that consist of both infusions and boluses
%   in the one-compartment model with first order elimination


global tbolus

V =12.6;  
CL = 2.16;  
[y, u] = lapl1comodel;
fy = vectorize(inline(char(y)))     %#ok<NOPRT>
fu = vectorize(inline(char(u)))     %#ok<NOPRT>
t = 0:0.01:100;
ffy = fy(CL, V, t);
ffu = fu(t);
subplot(3,1,1:2)
plot(t, ffy, 'LineWidth',2); 
title('Concentration Time Course','FontSize',12)
ylabel('Concentration [mg/L]','FontSize',12)
subplot(3,1,3)
plot(t, ffu, 'LineWidth', 2,'LineWidth',2); hold on;
y=(tbolus==tbolus)*100; 
stem(tbolus, y, '--r', 'LineWidth',2);
ylim([0  max(y)*1.1])
xlabel('Time [h]', 'FontSize',12)
ylabel('Dosing Regimen','FontSize',12)
legend('infusion','bolus')
print('-r900', '-dtiff', 'laplgraph'); 

end


function r = dosing
%DOSING Create a function with the full dosing history.
%   R = DOSING specifies a combined administration regimen: amounts. 
%   and times for infusion or bolus: Dose bolus times, |tbolus|, and 
%   infusion times, |tinfus|, should be global variables. It returns 
%   a symbolic function |R| that contains the full dosing history as 
%   (in analytical form).

syms D R t
global tbolus tinfus

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
D = 200; tbolus = [12 24 36  48 60 72 84 96];
R = 100; tinfus = [0 1; 2 3; 4 5;];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

r = 0;
% input of all bolus
for i = 1:length(tbolus)
    r = r + D*dirac(t-tbolus(i));       
end
% input of all infusions
for i = 1:length(tinfus)
    r = r + R*(heaviside(t-tinfus(i,1)) - heaviside(t-tinfus(i,2)));    
end

end


function [c,u] = lapl1comodel
%LAPL1COMODEL Solve one-compartment using Laplace transform.
%   [C,U] = LAPL1COMODEL solves one-compartment model equation, using
%   the Laplace transform function. Output: |C| - drug concentrations
%   as a function of time in analytical form (as response to the
%   dosing regimen) and |U| - the dosing as a function of time, in
%   analytical form, too.
  
syms  CL V  s t c u 

Ks = 1/(CL + V*s);     % transform function 
% specify the input (dosing regimen)
u = dosing;
Us = laplace(u, t, s);
Ys = Ks*Us;
c = ilaplace(Ys, s, t);
display(['c(t) = ' char(c)]);
pretty(c);

end





