function multiordose() 
%MULTIORDOSE One-compartment model with multiple oral administration.
%   MULTIORDOSE() calculates and produces a graph with concentration- 
%   time profiles for a one-compartment model with first-order drug 
%   absorption and first-order elimination, and multiple dose oral 
%   administration. Additionally, the superposition principle is 
%   illustrated with the plasma concentration profiles associated 
%   with each individual dose within the multiple oral 
%   administration.

p.k01 = 0.5;    % 1/h
p.V = 1.0;      % L   
p.CL = 2.0;     % L/h
p.F = 1;        % L/h
tdose = [0 1 2 3 4];
n = length(tdose);
Dose = 200*p.F*ones(1,n);   % mg 
timeEnd = 5;                % observation time
nd = 5;                     % number of doses
% Compute drug concentration directly
ti = [tdose timeEnd];       % add time after the last dose
tPoints = []; cPoints = [];
m0 = 0; c0= 0;
for i = 1:nd
    m0 = m0 + Dose(i); 
    tspan = [ti(i) ti(i+1)];
    [t,y] = ode45(@derivatives, tspan, [m0; c0], [], p);
    tPoints = [tPoints; t];         %#ok<AGROW>
    cPoints = [cPoints; y(:,2)];    %#ok<AGROW>
    m0 = y(end,1);  c0 = y(end,2);
end
% Graph preparation
ax = axes;
set(ax,'FontSize',15)
plot(tPoints, cPoints, 'o','LineWidth',1.5, 'Color', 'Black'); 
hold on;  grid on
title({'Superposition Principle', ...
    'One-Compartment Model with Oral Administration'})
ylabel('Drug concentration [mg/L]')
xlabel('Time [h]')
% Compute drug concentration using superposition principle (overall)
t = (0:0.01:timeEnd)';
c1  = ansoln(t, p, nd, Dose, tdose);
plot(t,c1,'LineWidth',3,'Color','red');
% Compute drug concentrations associated with doses
for idose = 1:nd
    ci  = ansoln(t, p, 1, Dose(idose), tdose(idose));
    plot(t,ci,'LineWidth', 2);
end
legend('concentration - direct', ... 
    'overall concentration - superposition', ... 
    'dose associated concentrations', 'Location', 'NorthWest')
print('-dtiff','-r900','multiordose')

end     
 

function c1 = ansoln(t, p, n, Dose, tdose)
%ANSOLN Apply the the superposition principle.
%   C1 = ANSOLN(T, P, N, DOSE, TDOSE) computes the concentration
%   values at time points |T|, with model parameters |P|. Dose
%   administration is defined by the vectors of dose amounts |DOSE|,
%   and dose times |TDOSE|, both of length |N|. The function is
%   recursive, i.e., it calls itself as long as |N| > 1.
   
k01 = p.k01;
k10 = p.CL/p.V;
F = p.F;
dt = t - tdose(n);
D = Dose(n);
if n == 1 
   c1 = F*D*k01/(k01-k10)*(exp(-k10*dt)-exp(-k01*dt)).*heaviside(dt); 
else
   c1 = ansoln(t, p, n-1, Dose, tdose) +  ...
        F*D*k01/(k01-k10)*(exp(-k10*dt)-exp(-k01*dt)).*heaviside(dt); 
end

end


function dydt = derivatives(~, y, p) 
%DERIVATIVES Compute the right-hand side of the ODE.
%   DYDT = DERIVATIVES(T, Y, P) calculates |DYDT|, the right-hand
%   side of the ODE model, at points defined by the vector of
%   dependent variables |Y|, time |T|, and with parameters |P|.

dy1dt = - p.k01*y(1);             
dy2dt = p.k01*y(1) - p.CL/p.V*y(2);
dydt = [dy1dt; dy2dt];

end
