function nlmeBaseAnalysis
%NLMEBASEANALYSIS Conduct a basic population analysis.
%   NLMEBASEANALYSIS computes the model estimates without the
%   consideration of covariates affecting the PK parameters.
%   It also produces some basic graphs of goodness of fit and
%   visual predictive checks. 

clear; clc; close all;
rng(0,'twister')        % for reproducibility
tic
nmData = load('nmData'); 
Data   = nmData.Data;
id     = Data(:,1);
time   = Data(:,2);
conc   = Data(:,3);
wt     = Data(:,4);
sex    = Data(:,5);
n    = length(unique(id));
dose = 100*ones(n,1);
wt   = grpstats(wt,id,@min);
sex  = grpstats(sex,id,@min);

%prepare input for nlmefit
nphi = 2;           %number of PK parameters 
P = [ 1 0; 0 1];    %covariance pattern for random effects 
%fixed effects matrix 
A = repmat(eye(nphi,nphi),[1,1,n]);             % NO COVARIATES 
%                                               % NO COVARIATES
%                                               % NO COVARIATES
b0 = [1; 1];                                    % NO COVARIATES

options = statset('OutputFcn',@nlmefitoutputfcn); 

% Calling Nonlinear Mixed-Effects(NLME) regression mode 
[beta,PSI,stats,b] = nlmefit(time,conc,id,dose,@modelana,b0,   ...
                    'Options', options,                        ...
                    'REParamsSelect',[1 2],                    ...
                    'ApproximationType', 'LME',                ...
                    'ErrorModel','Proportional',               ...
                    'CovPattern',P,                            ...      
                    'OptimFun','fminunc',                      ...
                    'ParamTransform',[1 1],                    ...
                    'FeGroupDesign',A)  %#ok<NOPRT>

% Calling Stochastic Approx. Expectation-Maximization mode(SAEM)
% [beta,PSI,stats,b] = nlmefitsa(time,conc,id,dose,@modelana,b0, ...  
%                     'Options', options,                        ...
%                     'REParamsSelect',[ 1 2],                   ...
%                     'LogLikMethod', 'is',                      ...
%                     'ErrorModel','Proportional',               ...
%                     'CovPattern',P,                            ...      
%                     'OptimFun','fminunc',                      ...
%                     'ParamTransform',[1 1],                    ...
%                     'FeGroupDesign',A)            %#ok<NOPRT>
beta        %#ok<NOPRT>
exp(beta)
PSI         %#ok<NOPRT>
stats       %#ok<NOPRT>
stats.covb
%print(figure(1),'-r600','-dtiff','baseiterations')       
toc
tic                
predTime(id,time,conc,beta,b,wt,sex)
predObs(id,time,conc,beta,b,wt,sex)

nrep = 1000;
vpc(nrep,n,time,conc,beta,PSI,stats.errorparam,wt,sex)
toc

end


function pred = modelana(phi,t,G)
%MODELANA Compute prediction values from the analytical model form.
%   PRED = MODELANA(PHI,T,G) calculates the predicted values values
%   |PRED| at observation points |T|, given model parameters |PHI|
%   and group specific predictor |G|. The prediction uses the
%   analytical model form.

dose = G;
pred = dose./phi(1)*exp(-phi(2).*t); 

end


function pred = modelvpc(phi,t,G,error)
%MODELVPC Compute model prediction values for the VPC
%   PRED = MODELVPC(PHI,T,G,ERROR) calculates predicted values 
%   |pred| at observation points |T|, given model parameters |phi|,   
%   and group specific predictor |G|. Residual error |error| is 
%   assumed. The values are needed for the visual prediction check
%   (VPC)

dose = G;
pred = dose./phi(1)*exp(-phi(2).*t) + ... 
    dose./phi(1)*exp(-phi(2).*t)*error;
end


function pred = modelODE(phi,t,G)         %#ok<DEFNU>
%MODELODE Compute model prediction values from the ODE model form
%   PRED = MODELODE(PHI,T,G) calculates the predicted values |PRED|
%   at observation points |T|, given model parameters |PHI| and group
%   and group specific predictor |G|. The prediction uses the
%   ODE model form.

dose = G;
options = odeset('RelTol',10^-3,'AbsTol',10^-3);
sol = ode113(@ode, [0,10], dose./phi(1), options);
pred = deval(sol,t)';

    % Call the ODE model 
    function dydt=ode(~,y)
        dydt = -phi(2)*y;
    end
end


function predTime(id,time,conc,beta,b,~,~)
%PREDTIME Create individual graphs for the goodness of fit.  
%   PREDTIME(ID,TIME,CONC,BETA,B,WT,SEX) generates individual graphs
%   with time courses (observations, population and individual 
%   predictions) for the goodness of fit purpose. The input: 
%   |ID| - subject numbers; |TIME| - observation times;
%   |CONC| - observed drug concentrations; |BETA| - model estimates;  
%   |B| - random effect values; |WT|,|SEX| - weight and sex as 
%   covariates.
%
%   This function uses suplabel from MATLAB Central:
%   http://www.mathworks.ch/matlabcentral/fileexchange/7772-suplabel
%   Copyright (c) 2004, Ben Barrowes
%   All rights reserved.
%   Code covered by the BSD License: http://www.mathworks.com/
%   matlabcentral/fileexchange/view_license?file_info_id=7772

display 'predTime'
n = length(unique(id));
d = repmat(beta(1:2), 1, n);
qq1   = d + [b(1,:); b(2,:)];
qq2   = qq1*0;                                  % NO COVARIATES
qq    = qq1 + qq2;
pkpar = exp(qq);
solu  = exp(d+qq2);

fig1 = figure;
for i=1:20
    subplot(5,4,i)
    semilogy(time, modelana(pkpar(:,i),time, 100),'-g')
    hold on
    semilogy(time(id==i),conc(id==i),'o');
    semilogy(time,modelana(solu(:,i),time,100),'-r')
    axis([0 6 10^-3 10^2])
end
[~,hx]=suplabel('Time');                        set(hx,'FontSize',14)
[~,hy]=suplabel('Drug Concentration','y');      set(hy,'FontSize',14)
[~,ht]=suplabel('Individual Time Courses','t'); set(ht,'FontSize',18)

print(fig1,'-dtiff','-r600','basefig1')

end


function predObs(id,time,conc,beta,b,~,~)
%PREDOBS Create a basic goodness of fit 
%   PREDOBS(ID,TIME,CONC,BETA,B,WT,SEX) generates a graph, observed
%   vs. predicted values) for the goodness of fit purpose. 
%   The input: |ID| - subjects numbers; |TIME| - observation times;
%   |CONC| - observed drug concentrations; |BETA| - model estimates;  
%   |B| - random effect values; |WT|,|SEX| - weight and sex as 
%   covariates. 

display 'predObs'
fig2 = figure;
n   = length(unique(id));
d   = repmat(beta(1:2), 1, n);
qq1 = d + [b(1,:); b(2,:)];
qq2 = qq1*0;                                    % NO COVARIATES
qq  = qq1 + qq2;
pkpar = exp(qq);
solu  = exp(d+qq2);
set(axes,'FontSize',15);
for i=1:n
    hp = loglog(modelana(solu(:,i),time(id==i),100), ... 
        conc(id==i),'r*');
    hold on
    hi = loglog(modelana(pkpar(:,i),time(id==i),100), ... 
        conc(id==i),'g*');
end
legend ([hp hi], 'population prediction','individual prediction')
xlabel('Predicted')
ylabel('Observed')
print(fig2,'-dtiff','-r600','basefig2')

end


function vpc(nrep,nsub,time,conc,beta,PSI,error,~,~)
%VPC conduct Visual Predictive Checks
%   VPC(NREP,NSUB,TIME,CONC,BETA,PSI,ERROR,WT,SEX) is a simulation 
%   method to evaluate if the model correctly describes the observed 
%   data, and if it can predict responses. The evaluation is based on 
%   the following input: |NREP| - replications(simulations) number; 
%   |NSUB| - number of subjects; |TIME| - observation times;
%   |CONC| - observed drug concentrations; |BETA|- model estimates; 
%   |PSI| - covariance matrix; |ERROR| - standard error of |BETA|;
%   |WT|,|SEX| - weight, sex as covariates 
%
%   This function uses plot_c from MATLAB Central:  
%   http://www.mathworks.com/matlabcentral/fileexchange/31752-plotci
%   Copyright (c) 2011, Zbigniew
%   All rights reserved.
%   Code covered by the BSD License: http://www.mathworks.com/
%   matlabcentral/fileexchange/view_license?file_info_id=31752

display 'vpc'

times = unique(time);
ntimes = length(times);
B1 = repmat(1:nrep,nsub*ntimes,1);
B2 = repmat((1:nsub),ntimes,1);
Bx = repmat(vertcat(B2(:)),nrep,1);
B3 = repmat(times,nrep*nsub,1);
BB = [B1(:) Bx B3(:)];
B4 = zeros(length(B3),1);

for k = 1:nrep
    vpc1 = mvnrnd(zeros(2,1),PSI,nsub);
    vpc2 = bsxfun(@plus,vpc1,[beta(1) beta(2)]);
%     vpc3 = vpc2 + [beta(3)*wt beta(4)*sex];   % INCLUDE COVARIATES
    vpc3 = vpc2;                              % NO COVARIATES
    for i = 1:nsub;
        pred = modelvpc(exp(vpc3(i,:)),times,100,error)';
        B4(((k-1)*nsub*ntimes+(i-1)*ntimes+1):((k-1)*nsub*ntimes+ ...
            (i-1)*ntimes+4)) = pred(:);
    end
end
A = [BB B4];
p5  = @(x) prctile(x,5);
p95 = @(x) prctile(x,95);
figVPC = figure;
set(axes,'FontSize',15)
semilogy(times, grpstats(conc,time,@median), 'sr', ...
    'MarkerFaceColor','r') 
hold on
semilogy(times,grpstats(conc,time,p5),'sb','MarkerFaceColor','b') 
semilogy(times,grpstats(conc,time,p95),'sk','MarkerFaceColor','k') 
legend('median', 'perc. 0.05', 'perc. 0.95') 
 
E5x = zeros(nrep*ntimes,3);
E50x = zeros(nrep*ntimes,3);
E95x = zeros(nrep*ntimes,3);
for i=1:nrep
    [E5,E50,E95] = grpstats(A(A(:,1)==i,[1,3,4]),A(A(:,1)==i,3), ...
        {p5 @median p95});
    E5x((i-1)*ntimes+1:(i-1)*ntimes+4,[1 2 3]) = E5;
    E50x((i-1)*ntimes+1:(i-1)*ntimes+4,[1 2 3]) = E50;
    E95x((i-1)*ntimes+1:(i-1)*ntimes+4,[1 2 3]) = E95;
end
% get 5% and 95% percentiles
p5all = grpstats([E5x(:,[2 3]) E50x(:,3) E95x(:,3)],E5x(:,2),p5);
p95all = grpstats([E5x(:,3) E50x(:,3) E95x(:,3)],E5x(:,2),p95);
vpc = [p5all p95all];
semilogy(vpc(:,1),vpc(:,2),'-b')
semilogy(vpc(:,1),vpc(:,5),'-b')
predCI = [vpc(:,2) vpc(:,5)];  
rgbfill = [171 205 255]/255;
% call plot_ci; the function is available on MATLAB Central: 
plot_ci(times, predCI, 'PatchColor', rgbfill, 'PatchAlpha', 0.5, ...
    'LineWidth', 1, 'LineStyle','-', 'LineColor', 'b');
semilogy(vpc(:,1),vpc(:,3),'-r')
semilogy(vpc(:,1),vpc(:,6),'-r')
predCI = [vpc(:,3) vpc(:,6)]; 
rgbfill = [255 180 180]/255; 
plot_ci(times, predCI, 'PatchColor', rgbfill, 'PatchAlpha', 0.5, ...
    'LineWidth', 1, 'LineStyle','-', 'LineColor', 'r');
semilogy(vpc(:,1),vpc(:,4),'-k')
semilogy(vpc(:,1),vpc(:,7),'-k')
predCI = [vpc(:,4) vpc(:,7)]; 
rgbfill = [200 200 200]/255; 
plot_ci(times, predCI, 'PatchColor', rgbfill, 'PatchAlpha', 0.5, ...
    'LineWidth', 1, 'LineStyle','-', 'LineColor', 'k');
xlabel('Time')
ylabel('Concentration')
print(figVPC,'-dtiff','-r600','finalVPC')

end 

