function nlmeData
%NLMEDATA Create a dataset with simulated data
%   NLMEDATA generates for a NLMEM analysis. The body weight 
%   distribution wtc is assumed to be uniform between 40 and 60,  
%   centered around 50. sex has a binomial distribution with p=0.5; 
%   0 coding for male, and 1 for female. The variability of the model
%   parameters id log-normal distributed, and the residuals - normal
%   distributed.

rng(0,'twister') % for reproducibility
n      = 40; 
times  = 0.5*[0; 1; 4; 8; ];
beta0  = log([10 0.8]); % V ke
psi    = [0.3 0; 0 0.1];
% random effects
xb     = mvnrnd(beta0,psi,n);
wtc    = 40+20*rand(n,1)-50; %centered weight
sex    = binornd(1,0.5,n,1);
xb(:,1)= xb(:,1)+0.2*wtc;  
xb(:,2)= xb(:,2)+0.9*sex;
phi    = exp(xb);
reserr = 0.1;

groups = repmat(1:n,numel(times),1);
groups = vertcat(groups(:));
y      = zeros(n*numel(times),1);
x      = zeros(n*numel(times),1);
for i=1:n
    idx    = groups == i;
    f      = model(phi(i,:),times); 
    % For ODE model form use the function: modelODE(...) 
    y(idx) = f + f*reserr.*randn(numel(times),1);
    x(idx) = times;
end

xx1  = repmat(wtc,1,length(times))'                 %#ok<NOPRT>
xx2  = repmat(sex,1,length(times))'                 %#ok<NOPRT>
Data = [groups x y vertcat(xx1(:)) vertcat(xx2(:))] %#ok<NOPRT>
dlmwrite('nm.dat',Data,'delimiter','\t', 'precision',3)
save('nmdata.mat','Data') 

end


function pred = model(phi,t)
%MODEL compute model prediction values from the analytical form
%   PRED = MODEL(PHI,T) calculates the predicted values |PRED| at
%   observation points |T|, given model parameters |PHI|

pred = 100./phi(1)*exp(-phi(2).*t);

end


function pred = modelODE(phi,t)   %#ok<DEFNU>
%MODELODE compute model prediction values from the ODE model form
%   PRED = MODELODE(PHI,T) calculates the predicted values 
%   |PRED| at observation points |T| and given model parameters |PHI|      

options = odeset('RelTol',10-9,'AbsTol',10-9);
sol = ode45(@ode,[0,10],100./phi(1),options);
pred = deval(sol,t)';

    % Call the ODE model 
    function dydt=ode(~,y)
        dydt = -phi(2)*y;
    end

end
