function  nlregr()
%NLREGR estimate parameters in PK model
%   %NLREGR() uses the  nonlinear least squares method to find model
%   parameter estimates
%
% This function uses plot_ci from MATLAB Central:  
% http://www.mathworks.com/matlabcentral/fileexchange/31752-plotci
% Copyright (c) 2011, Zbigniew
% All rights reserved.
% Code covered by the BSD License: http://www.mathworks.com/
% matlabcentral/fileexchange/view_license?file_info_id=31752
%

observations = [ 0  1  2   3   4   5   6   12       % time points    
                 0  1  1.5 1.6 1.5 1.4 1.3 1.4 ]';  % drug amount   
tObs = observations(:,1);  
aObs = observations(:,2);

% beta estimation  
options = statset('Robust', 'on', 'WgtFun', 'bisquare');
[beta, resid, J] = nlinfit(tObs, aObs, @model, [1 2], options); 
% objective function 
w = (abs(resid)<1).*(1 - resid.^2).^2;  % weight function 'bisquare'
ofv = resid'*diag(w)*resid;
% confidence interval for model parameters
paramCI = nlparci(beta,resid,'jacobian',J);
display([beta' paramCI]);
% confidence interval for predictions
tPred = 0:0.02:12;
[aPred,delta] = nlpredci(@model, tPred, beta, resid, 'jacobian', ... 
    J, 'simopt', 'on');
[~, iInd] = ismember(tObs,tPred);
% show predictions
display(['time', '        Obs', '        Pred']);
display(num2str([tObs aObs aPred(iInd)]))
predCI = [aPred aPred+delta aPred-delta];
set(axes,'FontSize',15)
plot(tObs, aObs ,'or', 'MarkerSize',5,'MarkerFaceColor','r'); 
grid on; hold on
rgbfill = [0.3047 0.6992 0.9102];
rgbline = [0.9414 0.9414 0.3359];
% call plot_ci; the function is available on MATLAB Central:  
plot_ci(tPred, predCI, 'PatchColor', rgbfill, 'PatchAlpha', 0.5, ...
    'MainLineWidth', 2, 'MainLineColor', rgbline, 'LineWidth', 1, ...
    'LineStyle','-', 'LineColor', 'k');
legend('observation', 'prediction CI','regression line', ...
    'Location','Best' );
xlabel('Time')
ylabel('amount')
title(['Fitting: ',' ka=', num2str(beta(1)),' k=', ...
     num2str(beta(2)), '  OFV(least squares)=', num2str(ofv)])
print('-r1000', '-dtiff', 'nlregression');

end


function dadt = derivatives(~, a, p)
%DERIVATIVES Compute the right-hand side of the ODE.
%   DADT = DERIVATIVES(T, A, P) calculates |DYDT|, |, the right-hand 
%   side of the ODE model, at points defined by the vector |A|,
%   time |T|, and with parameters |P|.

% PK one-compartment model (SC administration)   
dadt = [-p.ka*a(1);             % drug amount - depot 
         p.ka*a(1) - p.k*a(2)]; % drug amount - compartment 
end 


function a = model(beta, t)
%MODEL Prediction of the drug amount.
%   A = MODEL(BETA, T) computes prediction for the drug amount |A| 
%   in one-compartment model at time points given as vector |T|,
%   and with model parameters |BETA|.  
 
dose = 100;
amount0 = 0; 
tspan = [0 12];
a0 = [dose amount0];
p.ka = beta(1);     % absorption rate
p.k = beta(2);      % elimination rate
sol = ode45(@derivatives, tspan, a0,[], p);
a12 = deval(sol, t)';
a = a12(:,2);

end
