function pbpk4DAE
%PBPK4DAE Solve a PBPK model with 3 organs.
%   PBPK4DAE calculates the time courses of the drug concentration 
%   in blood and in a PBPK model representing blood and 3 organs 
%   (gut, liver, muscle), and produces the respective graphs. 
%   The DAE solver is applied.

% flows (gut, liver, muscle)
            p.QG = 2;    p.QL = 1;   p.QM = 1;
% volumes (blood, gut, liver, muscle)
p.Vb = 5;   p.VG = 4;    p.VL = 4;   p.VM = 4;    
% partition coefficients ( gut, liver, muscle)
            p.RG = 0.5;  p.RL = 2;   p.RM = 0.5;  
% initial values for organs (blood, gut, liver, muscle)
cbi = 0;  cGi = 10;  cLi = 0;  cMi = 0; 
% drug concentration for arterials (A) and venous (V)
cAi = cbi;   cVGi = cGi/p.RG;   cVLi = cLi/p.RL;   cVMi = cMi/p.RM;
cVi = ((p.QL + p.QG)*cVLi + p.QM*cVMi)/(p.QL + p.QG + p.QM);
% vector of initial values
y0 = [cbi; cGi; cLi; cMi; cAi; cVGi; cVLi; cVMi; cVi];
% specifiy Mass matrix
M = [ p.Vb 0    0    0    0    0    0    0    0                                                
      0    p.VG 0    0    0    0    0    0    0
      0    0    p.VL 0    0    0    0    0    0
      0    0    0    p.VM 0    0    0    0    0 
      0    0    0    0    0    0    0    0    0
      0    0    0    0    0    0    0    0    0
      0    0    0    0    0    0    0    0    0 
      0    0    0    0    0    0    0    0    0 
      0    0    0    0    0    0    0    0    0 ];                                              
options = odeset('Mass', M, 'RelTol',1e-9, 'AbsTol', 1e-9);
timeEnd = 24; tspan = [0 timeEnd]; 
% p.CL = 1;  
[t, y] = ode15s(@derivalgeb, tspan, y0, options, p);
set(axes,'FontSize', 15); plot(t,y(:, 1:4), 'LineWidth', 2);
hold on
% p.CL = 0;
% [t, y] = ode15s(derivalgeb, tspan, y0, options, p);
% plot(t,y(:, 1:4),'-.')
xlabel('Time [h]'); ylabel('Drug Concentration [mg/L]');
legend('Blood', 'Gut', 'Liver', 'Muscle'); title ('PBPK Model')
print('-dtiff','-r900','pbpk4DAE.tif')

end


function dydt = derivalgeb(~, y, p)
%DERIVALGEB Compute the right-hand side of the DAE.
%   DYDT = DERIVALGEB(T, Y, P) calculates |DYDT|, the right-hand
%   side of the DAE model at points defined by the vector of
%   dependent variables |Y|, time |T|, and with parameters |P|. 
%   The right-hand side represents both the differential equations 
%   and algebraic equations. 

% drug concentrations
cb = y(1);          % blood 
cG = y(2);          % gut
cL = y(3);          % liver
cM = y(4);          % muscle
% drug concentration in the arterials (A) and venous (V) 
cA  = y(5);          % 
cVG = y(6);         % gut
cVL = y(7);         % liver
cVM = y(8);         % muscle
cV  = y(9);         % muscle
% Mass balance
Q   = p.QL + p.QG + p.QM;
% rout = p.CL*c2;
% differential equations
dcbdt = Q*(cV - cA);
dcGdt = p.QG*(cA - cVG);
dcLdt = p.QL*cA + p.QG*cVG - (p.QL + p.QG)*cVL;
dcMdt = p.QM*cA - p.QM*cVM;
% algebraic equations
dcAdt  = cA  - cb;
dcVGdt = cVG - cG/p.RG;
dcVLdt = cVL - cL/p.RL;
dcVMdt = cVM - cM/p.RM;
dcVdt  = cV  - ((p.QL + p.QG)*cVL + p.QM*cVM)/Q;
% vector DAE = DE and AE 
dydt  = [dcbdt; dcGdt; dcLdt; dcMdt; ...
         dcAdt; dcVGdt; dcVLdt; dcVMdt; dcVdt];

end



