function pdepeHH
%PDEPEHH Simulate the action potential in a single neuron.
%   %PDEPEHH simulates a single neuron under deterministic 
%   stimulation using the full Hodgkin-Huxley model (in 2 dimensional
%   domain, i.e., time and distance). The model is described by a  
%   system of partial differential equations. To solve them, the  
%   MATLAB solver PDEPE is applied.

clear all

p = axonparams;

% steady state value
p.V0 = 0;        
p.n0 = 0.3177;   
p.m0 = 0.0529;  
p.h0 = 0.5961;  
p.tmax = 5;   
p.dt   = 0.01;
t = 0:p.dt:p.tmax; 
x = p.x;
msym = 0;         %symmetry parameter (slab)
p.h = waitbar(0, 'Please wait...');
sol = pdepe(msym, @pdehh, @pdeinic, @pdebouc, x, t, [], p);
close(p.h)
% Extract the first solution component as V(x,t)  
V = sol(:,:,1);
% Plot membrane potential V(t,x) as surface over time and distance 
solutionV = figure; set(axes, 'FontSize',15)
set(solutionV, 'Position',[190 50 560 420]);
surf(x, t, V, ... 
    'FaceColor','interp', 'EdgeColor','none','FaceLighting','phong');
colorbar; 
axis tight  
title('HH Model: PDE Solution V(x,t)');
xlabel('Distance [cm]');
ylabel('Time [ms]');
zlabel('Voltage [mV]');
print(solutionV, '-r600', '-dtiff', 'PDEsolutionVxt');

end


function p = axonparams
%AXONPARAMS setup axon parameters
%   P = AXONPARAMS creates a structure |P| with the physiological 
%   parameters of the neuron/axon

% Axon parameters
p.a    = 0.0238;       % axon radius                       [cm]
p.L    = 6;            % axon length                       [cm]
p.dx   = 0.05;         % distance integration step         [cm]  
p.x    = 0:p.dx:p.L;   % steps along axon                  [cm] 
p.lx   = length(p.x);  % number of steps within axon 
p.R    = 34.5;         % resistance (axoplasm)             [ohm*cm]
p.C    = 1;            % membrane capacitance              [uF/cm^2]
p.gK   = 36;           % max. potassium (K) conductance    [mS/cm^2]
p.gNa  = 120;          % max. sodium (Na) conductance      [mS/cm^2]
p.gL   = 0.3;          % leak (L) conductance              [mS/cm^2]
p.EK   = -12;          % K equilibrium potential           [mV] 
p.ENa  = 115;          % N equilibrium potential           [mV]
p.EL   = 10.598;       % L equilibrium potential           [mV]
p.Iinj = 10;           % injected stimulation current      [uA]
p.td   = 0.2;          % duration of injected current      [ms]
% p.td    = 20;        % duration for long stimulus        [ms]
p.T     = 18.5;                 % temperature              [Celsius]
% p.T    = 6.3;                 % temperature              [Celsius]
p.phi  = tempfactor(p.T);       % temperature factor

end


function [c,f,s] = pdehh(~, t, u, DuDx, p)
%PDEHH Evaluate the quantities defining the ODE system (HH Model).
%   [C,F,S] = PDEHH(X, T, U, DUDX, P) defines the form of the PDE 
%   (Hodgkin-Huxley model) according to the general transient 
%   equation required by MATLAB's pdepe. The input arguments are:
%   space variable |X|, time |T|, solution vector |U|, and its
%   partial derivative vector, |DUDX|, with respect to |X|. 
%   The structure |P| is used to pass additional parameters and 
%   other values required for the evaluation of |C|, |F| and |S|.
%   See also pdepe.

V = u(1);
n = u(2);
m = u(3);
h = u(4);

f = [p.a/(2*p.R)*1000; 0; 0; 0].*DuDx;
c = [p.C; 1; 1; 1];
s = [-(V-p.EK)*p.gK*n^4 - (V-p.ENa)*p.gNa*m^3*h - (V-p.EL)*p.gL ;
     nderiv(n, V, p.phi)
     mderiv(m, V, p.phi)
     hderiv(h, V, p.phi)]; 

% computations take place here
waitbar(t/p.tmax, p.h)
 
end


function u0 = pdeinic(~, p)
%PDEINIC Evaluate the initial conditions.
%   U0 = PDEINIC (X, P) returns the initial conditions vector |U0|,  
%   evaluated at point |X| (if not used here, indicated as |~|),  
%   and with consideration of additional parameters |P|.   

u0(1)= p.V0;
u0(2)= p.n0; 
u0(3)= p.m0;
u0(4)= p.h0;

end


function [pl,ql,pr,qr] = pdebouc(~, ~, ~, ur, t, p)  
%PDEBOUC Evaluate the boundary conditions.
%   [PL,QL,PR,QR] = PDEBOUC(XL, UL, XR, UR, T, P) returns quantities
%   |PL|, |QL|, |PR| and |QR| defining the boundary conditions  
%   evaluated at time |T|. |PL| and |QL| correspond to the left        
%   boundary point |XL| with the solution |UL|; |PR| and |QR|
%   correspond to the right boundary points |XR| with the solution  
%   |UR|. Additional parameters are passed using the structure |P|. 

Istim = p.Iinj*(heaviside(t)- heaviside(t - p.td));
pl = [Istim/(4*pi*p.a); 0; 0; 0];
ql = [1;                1; 1; 1];
pr = [ur(1);            0; 0; 0];
qr = [0;                1; 1; 1];

end


function dndt = nderiv(n, V, phi)
%NDERIV Time derivative of |N|.
%   DNDT = NDERIV(N, V, PHI) takes 2 arguments: |V| - membrane  
%   potential, |N| - concentration of proteins activating (opening) 
%   K channel and |PHI| - temperature factor. It returns the 
%   derivative value of |N|.

% Calculate values of the forward and backward rates: alpha, beta 
alpha = (0.01.*(-V + 10))./(exp((-V + 10)/10) - 1)*phi;
beta = 0.125*exp(-V/80)*phi;
dndt = alpha - (alpha + beta).*n;

end


function dmdt = mderiv(m, V, phi)
%DMDT Time derivative of |M|.
%   DMDT = MDERIV(M, V, PHI) takes 2 arguments: |V| - membrane  
%   potential, |M| - concentration of proteins activating (opening) 
%   Na channel and |PHI| - temperature factor. It returns the 
%   derivative value of |M|.  

% Calculate values of the forward and backward rates: alpha, beta
alpha = 0.1*(-V + 25)./(exp((-V + 25)/10) - 1)*phi;
beta = 4*exp(-V/18)*phi;
dmdt = alpha - (alpha + beta).*m;

end


function dhdt = hderiv(h, V, phi)
%HDERIV Time derivative of |H|.
%   DHDT = HDERIV(H, V, PHI) takes 2 arguments: |V| - membrane 
%   potential, |H| - concentration of proteins inactivating (closing)
%   Na channel and |PHI| - temperature factor. It returns the 
%   derivative value of |H|.

% Calculate values of the forward and backward rates: alpha, beta
alpha = 0.07*exp(-V/20)*phi;
beta = 1./(exp((-V + 30)/10) + 1)*phi;
dhdt = alpha - (alpha + beta).*h;

end


function phi = tempfactor(T)
%TEMPFACTOR Compute the temperature factor  
%   PHI = TEMPFACTOR(T) calculates the temperature factor |PHI| to  
%   be considered in HH model if the temperature |T| is different  
%   than 6.3 Celsius.   

phi = 3^((T-6.3)/10);

end

