function popAna
%POPANA Simultaneous analysis of data from several subjects.
%   POPANA shows different model-based approaches for analyzing of
%   PK-PD data from a population of subjects. As first, the observed
%   (simulated) data in four subjects are shown in a graph. Then,
%   three different approaches are conducted and graphically
%   illustrated: 
%   1. Pooled approach - one model is fitted to all data
%   without considering that data are grouped by subjects.
%   2. Two-stage approach - one model is fitted to each individual
%   data set separately and parameters are summarized. 
%   3. One-stage approach (population approach) - one model is fitted
%   to all data grouped by subjects.
%   The program is linked to Fig.6.2 (in the book).

clc; clear; close all;
% Prepare observations (simulate data)
subject = [ones(5,1);2*ones(5,1);3*ones(5,1);4*ones(5,1)];
time =[[0 1 3 5 9],[0 1 3 5 9],[0 1 3 5 9],[0 1 3 5 9]]';
rng(12321);
conc = 10*exp(-0.2*time)+normrnd(0,1,20,1);

% Observed Data
subplot(2,2,1)
gscatter(time, conc, subject)
xlabel('Time', 'FontSize', 12)
ylabel('Value', 'FontSize', 12)
axis([0 10 0 12])
title('Observed Data', 'FontSize', 12)

% Pooled Approach
subplot(2,2,2)
gscatter(time, conc)
hold on
model = @(phi,t)(phi(1)*exp(-phi(2)*t));
phi0 = [1 2];
[phi1,~] = nlinfit(time,conc,model,phi0);
phi1    %#ok<NOPRT>
tplot = 0:0.01:9;
plot(tplot,model(phi1,tplot),'k','LineWidth',2)
xlabel('Time', 'FontSize', 12)
ylabel('Value', 'FontSize', 12)
axis([0 10 0 12])
title('Pooled Approach', 'FontSize', 12)
hold off

% Two-Stage Approach
phi0 = [1 2];
PHI = zeros(2,4);
res = zeros(5,4);
for id = 1:4
    tI = time(subject == id);
    cI = conc(subject == id);
    [PHI(:,id),res(:,id)] = nlinfit(tI,cI,model,phi0);
end
PHI %#ok<NOPRT>
subplot(2,2,3)
gscatter(time,conc,subject)
hold on
colors = 'rgcbym';
for id = 1:4
    plot(tplot,model(PHI(:,id),tplot),'Color',colors(id))
end
xlabel('Time', 'FontSize', 12)
ylabel('Value', 'FontSize', 12)
axis([0 10 0 12])
title('Two-Stage Approach', 'FontSize', 12)
hold off

% One-Stage Approach
subplot(2,2,4)
gscatter(time,conc,subject)
hold on
nlme_model = @(PHI,t)(PHI(:,1).*exp(-PHI(:,2).*t));
phi0 = [1 2];
[phi2,~,~] = nlmefit(time,conc,subject,[],nlme_model,phi0);
phi2    %#ok<NOPRT>
plot(tplot,model(phi2,tplot),'k')
xlabel('Time', 'FontSize', 12)
ylabel('Value', 'FontSize', 12)
axis([0 10 0 12])
title('One-Stage Approach', 'FontSize', 12)

print('-dtiff','-r900','popAna')

end
