function simultaneous
%SIMULTANEOUS Conduct trials simultaneously. 

tic
ns = 500;
p = modelparams(ns);
ns = p.numSub;
a0 = 10*ones(ns,1); c0 = zeros(ns,1); y0 = [a0; c0]; tspan = [0 15];
[t,y] = ode15s(@derivatives, tspan, y0, [], p);
plot(t,y(:,ns+1 : 2*ns), 'Color', 'r'); hold on
toc
xlabel('Time'); ylabel('Concentration'); title('Simultaneous')
print('-r900', '-dtiff', 'simultaneous') 

end


function p = modelparams(ns)
%MODELPARAMS Specification of model parameters.
%   P = MODELPARAMS(NS) creates a structure |P| storing the model 
%   parameters for |NS| subjects.s.
 
% Reset the random number generator to the default
rng default
p.numSub = ns;                  % number of subjects
% specify model parameters for all subjects
p.ka = 1 + rand(ns,1);          % absorption rates 
p.V = 15 + rand(ns,1)*2;        % volume of distribution 
p.F = abs(1 - rand(ns,1)*0.5);  % bioavailability 
p.CL = 5 + rand(ns,1);          % clearance

end 


function dydt = derivatives(~, y, p)
%DERIVATIVES Compute the right-hand side of the ODE.
%   DYDT = DERIVATIVES(T,Y,P) calculates |DYDT|, the right-hand side
%   of the ODE model, at points defined by the vector of dependent
%   variable |Y|, time |T|, and with parameters |P|.
%
%   See also ode15s

ns = p.numSub;          % number of subjects
% set dependent variables simultaneously for all subjects 
% (vectorization)
a = y(   1 :   ns);     % drug amount                                
c = y(ns+1 : 2*ns);     % concentration                                  
% PK Model for all subjects
dadt = -p.ka.*a;
dcdt = (p.ka./p.V./p.F.*a - p.CL./p.V.*c);  
% Derivatives vector of ODE system
dydt = [ dadt        
         dcdt ];  
end
