function xnlmedrugccov(x)
%XNLMEDRUGCCOV Analysis of a two-compartment model with infusion.
%   XNLMEDRUGCCOV(X) conducts a NLMEM analysis of a two-compartment
%   model with infusion as described in Chap. 3. It includes the
%   covariates (weight and sex). The simulated dataset for the
%   analysis, drugCData.mat, is available. The |X| parameter must be
%   either 'estimation' or 'simulation'. The output of 'estimation'
%   is saved in the 'drugcparcov.mat' file, that can be used later
%   for 'simulation'.

load drugCData
rateGroup     = drugCData.RATE(drugCData.TIME==0);
wgtGroup      = drugCData.WGT(drugCData.TIME==0);
sexGroup      = drugCData.SEX(drugCData.TIME==0);
covarTable    = [rateGroup wgtGroup sexGroup];
pkIndex       = (drugCData.PKPD == 1) & (drugCData.TIME ~= 0);
time          = drugCData.TIME(pkIndex);
concentration = drugCData.DV(pkIndex);
subject       = drugCData.CID(pkIndex);
numSubjects   = length(unique(subject));
numSamples    = length(time);

numParams = 4;          % no. of model parameters
numBetas  = 8;          % no. of fixed efects 
numBrs    = 4;          % no. of random efects
% Initiate design matrices
A = zeros(numParams,numBetas,numSubjects);
B = diag(diag(ones(numParams)));    % diagonal B with '1's 
br00 = zeros(numParams,1); 

IDs =1:numSubjects;
for subjectID = IDs; 
    wgtnorm = wgtGroup(subjectID)/mean(wgtGroup);
    sex = sexGroup(subjectID);
    Acov = [wgtnorm 0       sex    0
            0       wgtnorm 0      sex
            0       0       0      0
            0       0       0      0  ];
    A(:,:,subjectID) = [eye(numParams) Acov];
end
reParams = [1 2 3 4];

switch x 
    case 'estimation'
        beta0 = [1.7448 1.0866 0.56138 1.2233 1  1 1 1];  
        PSI0 = [0.0001 0      0      0
                0      0.0001 0      0
                0      0      0.0001 0
                0      0      0      0.0001];                  
        covPattern = ones(numBrs);    
        xform = [1 1 1 1]; % transform for all parameters
        options = statset('Display','iter', 'MaxIter', 1000, ...
            'TolFun', 1e-3, 'TolX', 1e-3, 'OutputFcn', ... 
            @nlmefitoutputfcn );

        % Call SAEM procedure to fit nonlinear mixed effects model 
        [beta, PSI, stats, br] = nlmefitsa(time, concentration, ...
            subject, covarTable, @model, beta0, ...
            'ErrorModel', 'proportional',       ...
            'Options', options,                 ...
            'LogLikMethod', 'is',               ...
            'Cov0', PSI0,                       ...
            'CovPattern', covPattern,           ...  
            'ParamTransform', xform,            ...
            'NIterations', [150 150 100],       ...
            'OptimFun', 'fminunc',              ...
            'FEGroupDesign', A,                 ...
            'REParamsSelect', reParams)         %#ok<ASGLU,NOPRT>

        % Save for simulation
    save('drugcparcov.mat', 'beta', 'PSI', 'stats', 'br')
    case 'simulation'
        % Load estimated parameters
        load('drugcparcov.mat','beta', 'PSI', 'stats', 'br');
    otherwise
        %%%%%%%%%%%%%%%%
end

% POST PROCESSING

% add data to stats
stats.time = time;
stats.concentration = concentration;
stats.subject = subject;
stats.ipred = zeros(numSamples,1);
stats.ppred = zeros(numSamples,1);
stats.rate = zeros(numSamples,1);
for subjectID=IDs 
   stats.rate(subject==subjectID) = rateGroup(subjectID);   
end
stats  %#ok<NOPRT>

figSave.profilesFig = figure;
set(figSave.profilesFig, 'units', 'normalized', ...
    'Position', [0.05, 0.05, 0.45, 0.5]);
h = gscatter(time,concentration,subject,[],[],[],'off');
xlabel('time [hours]')
ylabel('concentration [mg/l]')
title('{\bf Drug-C Elimination}')
text(3,10,{'{\leftarrow 1.compartment:}',' concentrations vs. time'})
text(6, 5,{'{\leftarrow 2.compartment:}',' concentrations vs. time'})
T = max(time);

% Plot individual curves based on random effect estimates 
 
phirAll = zeros(numSubjects, numParams);       

for subjectID=IDs  
    br00(reParams) = br(1:numBrs,subjectID);    
    phir = A(:,:,subjectID)*beta + B*br00;    
    phir = phir';
    phir(1:numParams) = exp(phir(1:numParams)); % LOG
    [t, conc] = model(phir, T, covarTable(subjectID,:));
    line(t, conc,'color',get(h(subjectID),'color'))
    phirAll(subjectID,:) = br00(:);             
end

span = 0.5;
% clearance vs. weight
figSave.covFig = figure;
set(figSave.covFig, 'units', 'normalized', ...
    'Position', [0.05*11, 0.05, 0.45, 0.5]);
subplot(2,2,1);
gscatter(wgtGroup, phirAll(:,1),[],'k','.',[],'off')
hold on
phi1 = smooth(wgtGroup,phirAll(:,1),span,'rlowess');
[wgt,ind] = sort(wgtGroup);
line(wgt,phi1(ind)) 
xlabel('weight')
ylabel('\phi')
title('\phi(Clearance) ~ Weight ')

% volume vs. weight
subplot(2,2,3);  
gscatter(wgtGroup, phirAll(:,2),[],'k','.',[],'off')
hold on
phi1 = smooth(wgtGroup,phirAll(:,2),span,'rlowess');
[wgt,ind] = sort(wgtGroup);
line(wgt,phi1(ind)) 
xlabel('weight')
ylabel('\phi')
title('\phi(Volume) ~ Weight')

% clearance vs. sex
subplot(2,2,2);
boxplot(phirAll(:,1),sexGroup,'colors','k','symbol','ko')
xlabel('sex')
ylabel('\phi')
title('\phi(Clearance) ~ Sex')

% t-tests 

% volume vs. sex
subplot(2,2,4);
boxplot(phirAll(:,2),sexGroup,'colors','k','symbol','ko')
xlabel('sex')
ylabel('\phi')
title('\phi(Volume) ~ Sex')

figNr = 0;
for subjectID = IDs;
    tj = time(subject == subjectID);
    cj = concentration(subject == subjectID);
    graphNum = mod(subjectID-1,8) + 1;
    if graphNum == 1
        % create a new subplot (with 9 graphs)
        figNr = figNr + 1;
        figSave.individualFig(figNr) = figure; 
        set(figSave.individualFig(figNr), 'units', 'normalized', ...
            'Position', [0.05*figNr, 0.3, 0.50, 0.6]);
    end
    hs = subplot(3,3,graphNum);
    scatter(tj,cj,30,'black','filled');
    hold on
    xlabel('time [hours]')
    ylabel('concentration [mg/l]')
    title(['{\bf Subject: ' num2str(subjectID) '}'])
    % extract individual parameters
    br00(reParams) = br(1:numBrs,subjectID);    
    phir = A(:,:,subjectID)*beta + B*br00;     
    phir = phir';
    phir(1:numParams) = exp(phir(1:numParams)); % LOG
    [t, conc] = model(phir, T, covarTable(subjectID,:));  
    line(t,conc(:,1), 'linewidth', 1 , 'color','b', 'LineStyle', ':')
    % put individual predictions in stats-structure
    conc = model(phir, time(subject == subjectID), ... 
    covarTable(subjectID,:));
    stats.ipred(subject == subjectID) = conc;
    % extract population parameters
    phi = A(:,:,subjectID)*beta;
    phi = phi';
    phi(1:numParams) = exp(phi(1:numParams)); % LOG
    [t, conc, ~] = model(phi,T,covarTable(subjectID,:));
    line(t,conc(:,1), 'linewidth', 2 , 'color','b' )
    % put population predictions in stats-structure
    conc = model(phi, time(subject == subjectID), ... 
        covarTable(subjectID,:));
    stats.ppred(subject == subjectID) = conc;
    if graphNum == 8 || subjectID == numSubjects 
        leg =legend(hs, 'Observation', 'Individual Prediction', ... 
            'Population Prediction');
        set(leg, 'Position', [0.70 0.15 0.25 0.13]);
    end
end

figSave.gof1Fig = figure;  

subplot(2,2,1);
x = stats.time;    
y = stats.ppred;
plot(x,y,'.k', [0 15],[0 0],'-k')
title('Pop. Prediction vs. Time')
ylabel('Population Prediction');
xlabel('Time');
hold on

subplot(2,2,2);
x = stats.time;    
y = stats.ipred;
plot(x,y,'.k', [0 15],[0 0],'-k')
title('Ind. Prediction vs. Time')
ylabel('Individual Prediction');
xlabel('Time');
hold on

subplot(2,2,3);
x = stats.time;    
y = stats.pres;
plot(x,y,'.k', [0 15],[0 0],'-k')
title('Pop. Residual vs. Time')
ylabel('Population Residual');
xlabel('Time');
hold on

subplot(2,2,4);
x = stats.time;    
y = stats.ires;
plot(x,y,'.k', [0 15],[0 0],'-k')
title('Ind. Residual vs. Time')
ylabel('Individual Residual');
xlabel('Time');
hold on

figSave.gof2Fig = figure;
subplot(2,1,1);
for subjectID=IDs
    x = stats.time(subject==subjectID);    
    y = stats.pres(subject==subjectID);
    plot(x,y, '.k', [0 15],[0 0],'-k');
    title('Pop. Residual vs. Time')
    ylabel('Pop. Residual');
    xlabel('Time');
    hold on
end;

subplot(2,1,2);
for subjectID=IDs
    x = stats.time(subject==subjectID);    
    y = stats.ires(subject==subjectID);
    plot(x,y, '.k', [0 15],[0 0],'-k' );
    title('Individual Residual vs. Time')
    ylabel('Individual Residual');
    xlabel('Time');
    hold on
end;

figSave.gof3Fig = figure;
subplot(2,2,1);
plot(stats.ppred, stats.concentration,'.k');  hold on
plot(xlim,ylim,'-k');
axis('square');
ylabel('Observation');
xlabel('Population Prediction');

subplot(2,2,2);
plot(stats.ipred, stats.concentration,'.k');  hold on
plot(xlim,ylim,'-k');
axis('square');
ylabel('Observation');
xlabel('Individual Prediction');

subplot(2,2,3);
plot(stats.ppred, stats.cwres,'.k'); hold on
plot(xlim,[0 0],'-k');
axis('square');
ylabel('Conditional Weighted Residual');
xlabel('Population Prediction');

subplot(2,2,4);
plot(stats.ipred, stats.iwres,'.k'); hold on
plot(xlim,[0 0],'-k');
axis('square');
ylabel('Individual Weighted Residual');
xlabel('Population Prediction');

figSave.gof4Fig = figure;
subplot(2,1,1);
plot(stats.rate,stats.pwres,'.k'); hold on
plot(xlim,[0 0],'-k');
box off
xlabel('Rate');
ylabel('Population Weighted Residual');

subplot(2,1,2);
plot(stats.time,stats.pwres,'.k');  hold on
plot(xlim,[0 0],'-k')
box off
xlabel('Time');
ylabel('Population Weighted Residual');

printFig(figSave)

end

function [varargout] = model(phi, t, covar)
%MODEL compute model output values
%   C = MODEL(PHI,T,COVAR) calculates values of plasma concentration
%   |C| at observation time points |T|, given model parameters |PHI|
%   and covariates |COVAR|.
%
%   [TMOD,C] = MODEL(PHI, T, COVAR) calculates values of plasma
%   concentration |C| at observation interval of length |T|, given
%   model parameters |PHI| and covariates |COVAR|. The function
%   output consists of concentration values |C| at time points
%   |TMOD|.
%
%   [TMOD,C,P] = MODEL(PHI, T, COVAR) calculates values of plasma
%   concentration |C| at observation interval of length |T|, given
%   model parameters |PHI| and covariates |COVAR|. The function
%   output consists of concentration values |C| at time points
%   |TMOD|, structure |P| with model paramnetes

p.tR  = 2;           % infusion stop
p.R   = covar(1);    % rate
p.WGT = covar(2);    % weight as covariate
p.SEX = covar(3);    % weight as covariate

p.CL  = phi(1);      % central clearance
p.V1  = phi(2);      % volume of distribution in central compartment 
p.Q   = phi(3);      % inter-compartmental clearance
p.V2  = phi(4);      % volume of distribution peripheral compartment
p.k   = p.CL/p.V1;   % rate constant of elimination              
p.k12 = p.Q/p.V1;    % rate constant from central to peripheral             
p.k21 = p.Q/p.V2;    % rate constant from peripheral to central 

% Initial values
options = odeset('RelTol',1e-3, 'AbsTol',[1e-4 1e-4]);
tObs    = [0; t];       % 0 + obsevation times 
c0      = [0 0];          % Initial value for the dependant variable
if nargout == 1
    [~,y] = ode45(@derivatives, tObs, c0, options, p);
    conc = y(2:end,1);
    if max(isnan(conc)+ isinf(conc)) == 1
        % use penalty value as conc = NaN
        conc = 1e6; 
        display('penalty factor')
    end
    varargout(1) = {conc};
else 
    % solve during infusion - (interval 1) 
    tspan = [0 p.tR]; 
    c0    = [0 0];     % Initial value for the dependant variable
    [t0R, y0R] = ode45(@derivatives, tspan, c0, options, p);
    % solve during infusion - (interval 2)
    tspan = [t0R(end) t];
    c0    = y0R(end,:); % Initial value for the dependant variable
    [tRT, yRT] = ode45(@derivatives, tspan, c0, options, p);
    % (interval 2) + (interval 2)
    tmod = [t0R; tRT];
    conc = [y0R; yRT];
    if max(isnan(conc)+ isinf(conc)) == 1
        % use penalty value as conc = NaN
        conc = 1e6; 
        display('penalty factor')
    end    
    varargout(1) = {tmod};
    varargout(2) = {conc};
    varargout(3) = {p};
end

end

function dcdt = derivatives(t, c, p) 
%DERIVATIVES Compute the right-hand side of the ODE.
%   DCDT = DERIVATIVES(T, Y, P) calculates |DCDT|, the right-hand
%   side of the ODE model, at points defined by the vector of
%   dependent variables |Y|, time |T|, and with parameters |P|.
  
dcdt = [ R(t,p)/p.V1-(p.k+p.k12)*c(1)+p.k21*p.V2/p.V1*c(2); 
         p.k12*p.V1/p.V2*c(1)-p.k21*c(2)                    ]; 
end

function Rt = R(t,p)
%R Specifies Infusion Rate  
%   RT = R(T,P) returns infusion rate |RT| at time |T|, and with
%   parametrs |P|.

Rt = p.R * (t < p.tR);

end

function printFig(fig)
%PRINTFIG Print figures to files
%   PRINTFIG(FIG) saves in TIFF format graphs created during
%   simulation in external files. |FIG| is a structure containing
%   names of all figures to be saved/printed

print(fig.profilesFig, '-r600', '-dtiff', 'profFigcov');
print(fig.covFig, '-r600', '-dtiff', 'covFigcov');
for f = fig.individualFig
    x = find(f == fig.individualFig);
    fileName = ['IndProf', num2str(x), 'cov.tif'];
    print(f, '-r600', '-dtiff', fileName);
end
print(fig.gof1Fig, '-r600', '-dtiff', 'gof1Figcov');
print(fig.gof2Fig, '-r600', '-dtiff', 'gof2Figcov');
print(fig.gof3Fig, '-r600', '-dtiff', 'gof3Figcov');
print(fig.gof4Fig, '-r600', '-dtiff', 'gof4Figcov');

end

